"""
awslimitchecker/tests/services/test_base.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2018 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

from awslimitchecker.services.base import _AwsService
from awslimitchecker.limit import AwsLimit
from awslimitchecker.quotas import ServiceQuotasClient
import pytest
import sys
from datetime import datetime
from dateutil.tz import tzutc
from freezegun import freeze_time
from botocore.exceptions import ClientError

# https://code.google.com/p/mock/issues/detail?id=249
# py>=3.4 should use unittest.mock not the mock package on pypi
if (
        sys.version_info[0] < 3 or
        sys.version_info[0] == 3 and sys.version_info[1] < 4
):
    from mock import patch, call, Mock, PropertyMock
else:
    from unittest.mock import patch, call, Mock, PropertyMock


class AwsServiceTester(_AwsService):
    """class to test non-abstract methods on base class"""

    service_name = 'AwsServiceTester'
    api_name = 'awsservicetester'

    def connect(self):
        pass

    def find_usage(self):
        self._have_usage = True

    def get_limits(self):
        return {'foo': 'bar'}

    def required_iam_permissions(self):
        pass


class Test_AwsService(object):

    @pytest.mark.skipif(sys.version_info != (2, 7), reason='test for py27')
    def test_init_py27(self):
        with pytest.raises(TypeError) as excinfo:
            _AwsService(1, 2, {}, None)
        assert excinfo.value.message == "Can't instantiate abstract class " \
            "_AwsService with abstract methods " \
            "connect" \
            ", find_usage" \
            ", get_limits" \
            ", required_iam_permissions"

    @pytest.mark.skipif(sys.version_info < (3, 0), reason='test for py3')
    def test_init_py3(self):
        with pytest.raises(NotImplementedError) as excinfo:
            _AwsService(1, 2, {}, None)
        assert excinfo.value.args[0] == "abstract base class"

    def test_init_subclass(self):
        m_quota = Mock()
        cls = AwsServiceTester(1, 2, {}, m_quota)
        assert cls.warning_threshold == 1
        assert cls.critical_threshold == 2
        assert cls.limits == {'foo': 'bar'}
        assert cls.conn is None
        assert cls._have_usage is False
        assert cls._boto3_connection_kwargs == {}
        assert cls._quotas_client == m_quota
        assert cls._current_account_id is None
        assert cls._cloudwatch_client is None

    def test_init_subclass_boto_xargs(self):
        boto_args = {'region_name': 'myregion',
                     'aws_access_key_id': 'myaccesskey',
                     'aws_secret_access_key': 'mysecretkey',
                     'aws_session_token': 'mytoken'}

        cls = AwsServiceTester(1, 2, boto_args, None)
        assert cls.warning_threshold == 1
        assert cls.critical_threshold == 2
        assert cls.limits == {'foo': 'bar'}
        assert cls.conn is None
        assert cls._have_usage is False
        assert cls._boto3_connection_kwargs == boto_args
        assert cls._quotas_client is None
        assert cls._current_account_id is None
        assert cls._cloudwatch_client is None

    def test_current_account_id_stored(self):
        mock_conf = Mock(region_name='foo')
        mock_sts = Mock(_client_config=mock_conf)
        mock_sts.get_caller_identity.return_value = {
            'UserId': 'something',
            'Account': '123456789',
            'Arn': 'something'
        }
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        cls._current_account_id = '987654321'
        with patch('awslimitchecker.services.base.boto3.client') as m_boto:
            m_boto.return_value = mock_sts
            res = cls.current_account_id
        assert res == '987654321'
        assert m_boto.mock_calls == []

    def test_current_account_id_needed(self):
        mock_conf = Mock(region_name='foo')
        mock_sts = Mock(_client_config=mock_conf)
        mock_sts.get_caller_identity.return_value = {
            'UserId': 'something',
            'Account': '123456789',
            'Arn': 'something'
        }
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        with patch('awslimitchecker.services.base.boto3.client') as m_boto:
            m_boto.return_value = mock_sts
            res = cls.current_account_id
        assert res == '123456789'
        assert cls._current_account_id == '123456789'
        assert m_boto.mock_calls == [
            call('sts', foo='bar'),
            call().get_caller_identity()
        ]

    def test_set_limit_override(self):
        mock_limit = Mock(spec_set=AwsLimit)
        type(mock_limit).default_limit = 5
        cls = AwsServiceTester(1, 2, {}, None)
        cls.limits['foo'] = mock_limit
        cls.set_limit_override('foo', 10)
        assert mock_limit.mock_calls == [
            call.set_limit_override(10, override_ta=True)
        ]

    def test_set_limit_override_keyerror(self):
        mock_limit = Mock(spec_set=AwsLimit)
        type(mock_limit).default_limit = 5
        cls = AwsServiceTester(1, 2, {}, None)
        cls.limits['foo'] = mock_limit
        with pytest.raises(ValueError) as excinfo:
            cls.set_limit_override('bar', 10)

        if sys.version_info[0] > 2:
            msg = excinfo.value.args[0]
        else:
            msg = excinfo.value.message

        assert msg == "AwsServiceTester service has no " \
            "'bar' limit"
        assert mock_limit.mock_calls == []

    def test_set_ta_limit(self):
        mock_limit = Mock(spec_set=AwsLimit)
        type(mock_limit).default_limit = 5
        cls = AwsServiceTester(1, 2, {}, None)
        cls.limits['foo'] = mock_limit
        cls._set_ta_limit('foo', 10)
        assert mock_limit.mock_calls == [
            call._set_ta_limit(10)
        ]

    def test_set_ta_limit_keyerror(self):
        mock_limit = Mock(spec_set=AwsLimit)
        type(mock_limit).default_limit = 5
        cls = AwsServiceTester(1, 2, {}, None)
        cls.limits['foo'] = mock_limit
        with pytest.raises(ValueError) as excinfo:
            cls._set_ta_limit('bar', 10)

        if sys.version_info[0] > 2:
            msg = excinfo.value.args[0]
        else:
            msg = excinfo.value.message

        assert msg == "AwsServiceTester service has no " \
            "'bar' limit"
        assert mock_limit.mock_calls == []

    def test_set_threshold_override(self):
        mock_limit = Mock(spec_set=AwsLimit)
        type(mock_limit).default_limit = 5
        cls = AwsServiceTester(1, 2, {}, None)
        cls.limits['foo'] = mock_limit
        cls.set_threshold_override(
            'foo',
            warn_percent=10,
            warn_count=12,
            crit_percent=14,
            crit_count=16
        )
        assert mock_limit.mock_calls == [
            call.set_threshold_override(
                warn_percent=10,
                warn_count=12,
                crit_percent=14,
                crit_count=16
            )
        ]

    def test_set_threshold_override_keyerror(self):
        mock_limit = Mock(spec_set=AwsLimit)
        type(mock_limit).default_limit = 5
        cls = AwsServiceTester(1, 2, {}, None)
        cls.limits['foo'] = mock_limit
        with pytest.raises(ValueError) as excinfo:
            cls.set_threshold_override('bar', warn_percent=10)

        if sys.version_info[0] > 2:
            msg = excinfo.value.args[0]
        else:
            msg = excinfo.value.message

        assert msg == "AwsServiceTester service has no " \
            "'bar' limit"
        assert mock_limit.mock_calls == []

    def test_check_thresholds(self):
        cls = AwsServiceTester(1, 2, {}, None)
        cls.find_usage()
        mock_limit1 = Mock(spec_set=AwsLimit)
        mock_limit1.check_thresholds.return_value = False
        cls.limits['foo'] = mock_limit1
        mock_limit2 = Mock(spec_set=AwsLimit)
        mock_limit2.check_thresholds.return_value = True
        cls.limits['foo2'] = mock_limit2
        mock_limit3 = Mock(spec_set=AwsLimit)
        mock_limit3.check_thresholds.return_value = True
        cls.limits['foo3'] = mock_limit3
        mock_limit4 = Mock(spec_set=AwsLimit)
        mock_limit4.check_thresholds.return_value = False
        cls.limits['foo4'] = mock_limit4
        mock_find_usage = Mock()
        with patch.object(AwsServiceTester, 'find_usage', mock_find_usage):
            res = cls.check_thresholds()
        assert mock_limit1.mock_calls == [call.check_thresholds()]
        assert mock_limit2.mock_calls == [call.check_thresholds()]
        assert mock_limit3.mock_calls == [call.check_thresholds()]
        assert mock_limit4.mock_calls == [call.check_thresholds()]
        assert res == {'foo': mock_limit1, 'foo4': mock_limit4}
        assert mock_find_usage.mock_calls == []

    def test_check_thresholds_find_usage(self):
        cls = AwsServiceTester(1, 2, {}, None)
        mock_limit1 = Mock(spec_set=AwsLimit)
        mock_limit1.check_thresholds.return_value = False
        cls.limits['foo'] = mock_limit1
        mock_limit2 = Mock(spec_set=AwsLimit)
        mock_limit2.check_thresholds.return_value = True
        cls.limits['foo2'] = mock_limit2
        mock_limit3 = Mock(spec_set=AwsLimit)
        mock_limit3.check_thresholds.return_value = True
        cls.limits['foo3'] = mock_limit3
        mock_limit4 = Mock(spec_set=AwsLimit)
        mock_limit4.check_thresholds.return_value = False
        cls.limits['foo4'] = mock_limit4
        mock_find_usage = Mock()
        with patch.object(AwsServiceTester, 'find_usage', mock_find_usage):
            res = cls.check_thresholds()
        assert mock_limit1.mock_calls == [call.check_thresholds()]
        assert mock_limit2.mock_calls == [call.check_thresholds()]
        assert mock_limit3.mock_calls == [call.check_thresholds()]
        assert mock_limit4.mock_calls == [call.check_thresholds()]
        assert res == {'foo': mock_limit1, 'foo4': mock_limit4}
        assert mock_find_usage.mock_calls == [call()]

    def test_update_service_quotas(self):

        def se_get_quota_value(_, quota_name, **kwargs):
            if quota_name == 'qn1':
                return 12.4
            return None

        mock_client = Mock(spec_set=ServiceQuotasClient)
        mock_client.get_quota_value.side_effect = se_get_quota_value
        mock_limit1 = Mock(spec_set=AwsLimit)
        type(mock_limit1).quotas_service_code = PropertyMock(
            return_value='qsc'
        )
        type(mock_limit1).quota_name = PropertyMock(return_value='qn1')
        type(mock_limit1).quotas_unit = PropertyMock(return_value='None')
        type(mock_limit1).quotas_unit_converter = PropertyMock(
            return_value=None
        )
        mock_limit2 = Mock(spec_set=AwsLimit)
        type(mock_limit2).quotas_service_code = PropertyMock(
            return_value='qsc'
        )
        type(mock_limit2).quota_name = PropertyMock(return_value='qn2')
        type(mock_limit2).quotas_unit = PropertyMock(return_value='None')
        type(mock_limit2).quotas_unit_converter = PropertyMock(
            return_value=None
        )
        mock_limit3 = Mock(spec_set=AwsLimit)
        type(mock_limit3).quotas_service_code = PropertyMock(
            return_value='qsc'
        )
        type(mock_limit3).quota_name = PropertyMock(return_value='qn3')
        type(mock_limit3).quotas_unit = PropertyMock(return_value='Foo')
        mock_conv = Mock()
        type(mock_limit3).quotas_unit_converter = PropertyMock(
            return_value=mock_conv
        )
        cls = AwsServiceTester(1, 2, {}, mock_client)
        cls.quotas_service_code = 'qsc'
        cls.limits = {
            'limit1': mock_limit1,
            'limit2': mock_limit2,
            'limit3': mock_limit3
        }
        cls._update_service_quotas()
        assert mock_client.mock_calls == [
            call.get_quota_value('qsc', 'qn1', units='None', converter=None),
            call.get_quota_value('qsc', 'qn2', units='None', converter=None),
            call.get_quota_value(
                'qsc', 'qn3', units='Foo', converter=mock_conv
            )
        ]
        assert mock_limit1.mock_calls == [
            call._set_quotas_limit(12.4)
        ]
        assert mock_limit2.mock_calls == []

    def test_update_service_quotas_no_code(self):

        def se_get_quota_value(_, quota_name, **kwargs):
            if quota_name == 'qn1':
                return 12.4
            return None

        mock_client = Mock(spec_set=ServiceQuotasClient)
        mock_client.get_quota_value.side_effect = se_get_quota_value
        mock_limit1 = Mock(spec_set=AwsLimit)
        type(mock_limit1).quotas_service_code = PropertyMock(
            return_value='qsc'
        )
        type(mock_limit1).quota_name = PropertyMock(return_value='qn1')
        type(mock_limit1).quotas_unit = PropertyMock(return_value='None')
        mock_limit2 = Mock(spec_set=AwsLimit)
        type(mock_limit2).quotas_service_code = PropertyMock(
            return_value='qsc'
        )
        type(mock_limit2).quota_name = PropertyMock(return_value='qn2')
        type(mock_limit2).quotas_unit = PropertyMock(return_value='None')
        cls = AwsServiceTester(1, 2, {}, mock_client)
        cls.quotas_service_code = None
        cls.limits = {'limit1': mock_limit1, 'limit2': mock_limit2}
        cls._update_service_quotas()
        assert mock_client.mock_calls == []
        assert mock_limit1.mock_calls == []
        assert mock_limit2.mock_calls == []

    def test_update_service_quotas_no_client(self):

        def se_get_quota_value(_, quota_name, **kwargs):
            if quota_name == 'qn1':
                return 12.4
            return None

        mock_client = Mock(spec_set=ServiceQuotasClient)
        mock_client.get_quota_value.side_effect = se_get_quota_value
        mock_limit1 = Mock(spec_set=AwsLimit)
        type(mock_limit1).quotas_service_code = PropertyMock(
            return_value='qsc'
        )
        type(mock_limit1).quota_name = PropertyMock(return_value='qn1')
        type(mock_limit1).quotas_unit = PropertyMock(return_value='None')
        mock_limit2 = Mock(spec_set=AwsLimit)
        type(mock_limit2).quotas_service_code = PropertyMock(
            return_value='qsc'
        )
        type(mock_limit2).quota_name = PropertyMock(return_value='qn2')
        type(mock_limit2).quotas_unit = PropertyMock(return_value='None')
        cls = AwsServiceTester(1, 2, {}, None)
        cls.quotas_service_code = 'qsc'
        cls.limits = {'limit1': mock_limit1, 'limit2': mock_limit2}
        cls._update_service_quotas()
        assert mock_client.mock_calls == []
        assert mock_limit1.mock_calls == []
        assert mock_limit2.mock_calls == []

    def test_cloudwatch_connection_needed(self):
        mock_conf = Mock(region_name='foo')
        mock_cw = Mock(_client_config=mock_conf)
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        assert cls._cloudwatch_client is None
        with patch('awslimitchecker.services.base.boto3.client') as m_boto:
            m_boto.return_value = mock_cw
            res = cls._cloudwatch_connection()
        assert res == mock_cw
        assert cls._cloudwatch_client == mock_cw
        assert m_boto.mock_calls == [
            call.client('cloudwatch', foo='bar')
        ]

    def test_cloudwatch_connection_needed_max_retries(self):
        mock_conf = Mock(region_name='foo')
        mock_cw = Mock(_client_config=mock_conf)
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        assert cls._cloudwatch_client is None
        with patch('awslimitchecker.services.base.boto3.client') as m_boto:
            with patch(
                'awslimitchecker.connectable.Connectable._max_retries_config',
                new_callable=PropertyMock
            ) as m_mrc:
                m_mrc.return_value = {'retries': 5}
                m_boto.return_value = mock_cw
                res = cls._cloudwatch_connection()
        assert res == mock_cw
        assert cls._cloudwatch_client == mock_cw
        assert m_boto.mock_calls == [
            call.client('cloudwatch', foo='bar', config={'retries': 5})
        ]

    def test_cloudwatch_connection_stored(self):
        mock_conf = Mock(region_name='foo')
        mock_cw = Mock(_client_config=mock_conf)
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        cls._cloudwatch_client = mock_cw
        with patch('awslimitchecker.services.base.boto3.client') as m_boto:
            m_boto.return_value = mock_cw
            res = cls._cloudwatch_connection()
        assert res == mock_cw
        assert cls._cloudwatch_client == mock_cw
        assert m_boto.mock_calls == []


class TestGetCloudwatchUsageLatest:

    @freeze_time("2020-09-22 12:26:00", tz_offset=0)
    def test_defaults(self):
        mock_conn = Mock()
        mock_conn.get_metric_data.return_value = {
            'MetricDataResults': [
                {
                    'Id': 'id',
                    'Label': 'ResourceCount',
                    'Timestamps': [
                        datetime(2020, 9, 22, 12, 25, tzinfo=tzutc())
                    ],
                    'Values': [3.0],
                    'StatusCode': 'PartialData'
                }
            ],
            'NextToken': 'foo',
            'Messages': [],
            'ResponseMetadata': {
                'RequestId': '77a86d3c-16e1-47c5-b2d1-0f7d81d48a05',
                'HTTPStatusCode': 200,
                'HTTPHeaders': {
                    'x-amzn-requestid': '70f7d81d48a05',
                    'content-type': 'text/xml',
                    'content-length': '796',
                    'date': 'Tue, 22 Sep 2020 12:26:36 GMT'
                },
                'RetryAttempts': 0
            }
        }
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        with patch(
            'awslimitchecker.services.base._AwsService._cloudwatch_connection',
            autospec=True
        ) as m_cw_conn:
            m_cw_conn.return_value = mock_conn
            res = cls._get_cloudwatch_usage_latest([
                {'Name': 'foo', 'Value': 'bar'},
                {'Name': 'baz', 'Value': 'blam'}
            ])
        assert res == 3.0
        assert m_cw_conn.mock_calls == [
            call(cls)
        ]
        assert mock_conn.mock_calls == [
            call.get_metric_data(
                MetricDataQueries=[
                    {
                        'Id': 'id',
                        'MetricStat': {
                            'Metric': {
                                'Namespace': 'AWS/Usage',
                                'MetricName': 'ResourceCount',
                                'Dimensions': [
                                    {'Name': 'foo', 'Value': 'bar'},
                                    {'Name': 'baz', 'Value': 'blam'}
                                ]
                            },
                            'Period': 60,
                            'Stat': 'Average'
                        }
                    }
                ],
                StartTime=datetime(2020, 9, 22, 11, 26, 00),
                EndTime=datetime(2020, 9, 22, 12, 26, 00),
                ScanBy='TimestampDescending',
                MaxDatapoints=1
            )
        ]

    @freeze_time("2020-09-22 12:26:00", tz_offset=0)
    def test_non_default(self):
        mock_conn = Mock()
        mock_conn.get_metric_data.return_value = {
            'MetricDataResults': [
                {
                    'Id': 'id',
                    'Label': 'ResourceCount',
                    'Timestamps': [
                        datetime(2020, 9, 22, 12, 25, tzinfo=tzutc())
                    ],
                    'Values': [3.0],
                    'StatusCode': 'PartialData'
                }
            ],
            'NextToken': 'foo',
            'Messages': [],
            'ResponseMetadata': {
                'RequestId': '77a86d3c-16e1-47c5-b2d1-0f7d81d48a05',
                'HTTPStatusCode': 200,
                'HTTPHeaders': {
                    'x-amzn-requestid': '70f7d81d48a05',
                    'content-type': 'text/xml',
                    'content-length': '796',
                    'date': 'Tue, 22 Sep 2020 12:26:36 GMT'
                },
                'RetryAttempts': 0
            }
        }
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        with patch(
            'awslimitchecker.services.base._AwsService._cloudwatch_connection',
            autospec=True
        ) as m_cw_conn:
            m_cw_conn.return_value = mock_conn
            res = cls._get_cloudwatch_usage_latest([
                {'Name': 'foo', 'Value': 'bar'},
                {'Name': 'baz', 'Value': 'blam'}
            ], metric_name='MyMetric', period=3600)
        assert res == 3.0
        assert m_cw_conn.mock_calls == [
            call(cls)
        ]
        assert mock_conn.mock_calls == [
            call.get_metric_data(
                MetricDataQueries=[
                    {
                        'Id': 'id',
                        'MetricStat': {
                            'Metric': {
                                'Namespace': 'AWS/Usage',
                                'MetricName': 'MyMetric',
                                'Dimensions': [
                                    {'Name': 'foo', 'Value': 'bar'},
                                    {'Name': 'baz', 'Value': 'blam'}
                                ]
                            },
                            'Period': 3600,
                            'Stat': 'Average'
                        }
                    }
                ],
                StartTime=datetime(2020, 9, 22, 11, 26, 00),
                EndTime=datetime(2020, 9, 22, 12, 26, 00),
                ScanBy='TimestampDescending',
                MaxDatapoints=1
            )
        ]

    @freeze_time("2020-09-22 12:26:00", tz_offset=0)
    def test_exception(self):
        mock_conn = Mock()
        mock_conn.get_metric_data.side_effect = ClientError(
            {
                'ResponseMetadata': {
                    'HTTPStatusCode': 503,
                    'RequestId': '7d74c6f0-c789-11e5-82fe-a96cdaa6d564'
                },
                'Error': {
                    'Message': 'Service Unavailable',
                    'Code': '503'
                }
            },
            'GetMetricData'
        )
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        with patch(
            'awslimitchecker.services.base._AwsService._cloudwatch_connection',
            autospec=True
        ) as m_cw_conn:
            m_cw_conn.return_value = mock_conn
            res = cls._get_cloudwatch_usage_latest([
                {'Name': 'foo', 'Value': 'bar'},
                {'Name': 'baz', 'Value': 'blam'}
            ], metric_name='MyMetric', period=3600)
        assert res == 0
        assert m_cw_conn.mock_calls == [
            call(cls)
        ]
        assert mock_conn.mock_calls == [
            call.get_metric_data(
                MetricDataQueries=[
                    {
                        'Id': 'id',
                        'MetricStat': {
                            'Metric': {
                                'Namespace': 'AWS/Usage',
                                'MetricName': 'MyMetric',
                                'Dimensions': [
                                    {'Name': 'foo', 'Value': 'bar'},
                                    {'Name': 'baz', 'Value': 'blam'}
                                ]
                            },
                            'Period': 3600,
                            'Stat': 'Average'
                        }
                    }
                ],
                StartTime=datetime(2020, 9, 22, 11, 26, 00),
                EndTime=datetime(2020, 9, 22, 12, 26, 00),
                ScanBy='TimestampDescending',
                MaxDatapoints=1
            )
        ]

    @freeze_time("2020-09-22 12:26:00", tz_offset=0)
    def test_no_data(self):
        mock_conn = Mock()
        mock_conn.get_metric_data.return_value = {
            'MetricDataResults': [],
            'NextToken': 'foo',
            'Messages': [],
            'ResponseMetadata': {
                'RequestId': '77a86d3c-16e1-47c5-b2d1-0f7d81d48a05',
                'HTTPStatusCode': 200,
                'HTTPHeaders': {
                    'x-amzn-requestid': '70f7d81d48a05',
                    'content-type': 'text/xml',
                    'content-length': '796',
                    'date': 'Tue, 22 Sep 2020 12:26:36 GMT'
                },
                'RetryAttempts': 0
            }
        }
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        with patch(
            'awslimitchecker.services.base._AwsService._cloudwatch_connection',
            autospec=True
        ) as m_cw_conn:
            m_cw_conn.return_value = mock_conn
            res = cls._get_cloudwatch_usage_latest([
                {'Name': 'foo', 'Value': 'bar'},
                {'Name': 'baz', 'Value': 'blam'}
            ], metric_name='MyMetric', period=3600)
        assert res == 0
        assert m_cw_conn.mock_calls == [
            call(cls)
        ]
        assert mock_conn.mock_calls == [
            call.get_metric_data(
                MetricDataQueries=[
                    {
                        'Id': 'id',
                        'MetricStat': {
                            'Metric': {
                                'Namespace': 'AWS/Usage',
                                'MetricName': 'MyMetric',
                                'Dimensions': [
                                    {'Name': 'foo', 'Value': 'bar'},
                                    {'Name': 'baz', 'Value': 'blam'}
                                ]
                            },
                            'Period': 3600,
                            'Stat': 'Average'
                        }
                    }
                ],
                StartTime=datetime(2020, 9, 22, 11, 26, 00),
                EndTime=datetime(2020, 9, 22, 12, 26, 00),
                ScanBy='TimestampDescending',
                MaxDatapoints=1
            )
        ]

    @freeze_time("2020-09-22 12:26:00", tz_offset=0)
    def test_no_values(self):
        mock_conn = Mock()
        mock_conn.get_metric_data.return_value = {
            'MetricDataResults': [
                {
                    'Id': 'id',
                    'Label': 'ResourceCount',
                    'Timestamps': [],
                    'Values': [],
                    'StatusCode': 'PartialData'
                }
            ],
            'NextToken': 'foo',
            'Messages': [],
            'ResponseMetadata': {
                'RequestId': '77a86d3c-16e1-47c5-b2d1-0f7d81d48a05',
                'HTTPStatusCode': 200,
                'HTTPHeaders': {
                    'x-amzn-requestid': '70f7d81d48a05',
                    'content-type': 'text/xml',
                    'content-length': '796',
                    'date': 'Tue, 22 Sep 2020 12:26:36 GMT'
                },
                'RetryAttempts': 0
            }
        }
        cls = AwsServiceTester(1, 2, {'foo': 'bar'}, None)
        with patch(
            'awslimitchecker.services.base._AwsService._cloudwatch_connection',
            autospec=True
        ) as m_cw_conn:
            m_cw_conn.return_value = mock_conn
            res = cls._get_cloudwatch_usage_latest([
                {'Name': 'foo', 'Value': 'bar'},
                {'Name': 'baz', 'Value': 'blam'}
            ], metric_name='MyMetric', period=3600)
        assert res == 0
        assert m_cw_conn.mock_calls == [
            call(cls)
        ]
        assert mock_conn.mock_calls == [
            call.get_metric_data(
                MetricDataQueries=[
                    {
                        'Id': 'id',
                        'MetricStat': {
                            'Metric': {
                                'Namespace': 'AWS/Usage',
                                'MetricName': 'MyMetric',
                                'Dimensions': [
                                    {'Name': 'foo', 'Value': 'bar'},
                                    {'Name': 'baz', 'Value': 'blam'}
                                ]
                            },
                            'Period': 3600,
                            'Stat': 'Average'
                        }
                    }
                ],
                StartTime=datetime(2020, 9, 22, 11, 26, 00),
                EndTime=datetime(2020, 9, 22, 12, 26, 00),
                ScanBy='TimestampDescending',
                MaxDatapoints=1
            )
        ]


class Test_AwsServiceSubclasses(object):

    def test_subclass_init(self, cls):
        # ensure we set limits in the constructor
        mock_limits = Mock()
        mock_get_limits = Mock()
        mock_get_limits.return_value = mock_limits
        mock_quotas = Mock()
        with patch.object(cls, 'get_limits', mock_get_limits):
            inst = cls(3, 7, {}, mock_quotas)
        assert inst.limits == mock_limits
        assert inst._quotas_client == mock_quotas
        assert mock_get_limits.mock_calls == [call()]
        # ensure service name is changed
        assert inst.service_name != 'baseclass'
        assert inst.api_name != 'baseclass'
        # ensure an IAM permissions list, even if empty
        assert isinstance(inst.required_iam_permissions(), list)
        # ensure warning and critical thresholds
        assert inst.warning_threshold == 3
        assert inst.critical_threshold == 7
        assert not inst._boto3_connection_kwargs
        assert inst._current_account_id is None
        assert inst._cloudwatch_client is None

        boto_args = dict(region_name='myregion',
                         aws_access_key_id='myaccesskey',
                         aws_secret_access_key='mysecretkey',
                         aws_session_token='mytoken')

        sts_inst = cls(3, 7, boto_args, mock_quotas)
        assert sts_inst._boto3_connection_kwargs == boto_args
        assert sts_inst._quotas_client == mock_quotas



"""
awslimitchecker/services/cloudtrail.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2018 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

import sys

from awslimitchecker.tests.services import result_fixtures
from awslimitchecker.limit import AwsLimit
from awslimitchecker.services.cloudtrail import _CloudTrailService

# https://code.google.com/p/mock/issues/detail?id=249
# py>=3.4 should use unittest.mock not the mock package on pypi
if (
        sys.version_info[0] < 3 or
        sys.version_info[0] == 3 and sys.version_info[1] < 4
):
    from mock import patch, Mock
else:
    from unittest.mock import patch, Mock

module_base = 'awslimitchecker.services.cloudtrail.'
PATCH_BASE = '%s_CloudTrailService' % module_base
AWS_TYPE = "AWS::CloudTrail::Trail"


class Test_CloudTrailService(object):

    def test_init(self):
        with patch('%s.get_limits' % PATCH_BASE):
            cls = _CloudTrailService(21, 43, {}, None)
        assert cls.service_name == 'CloudTrail'
        assert cls.api_name == 'cloudtrail'
        assert cls.conn is None
        assert cls.resource_conn is None
        assert cls.warning_threshold == 21
        assert cls.critical_threshold == 43

    def test_get_limits(self):
        cls = _CloudTrailService(21, 43, {}, None)

        limit_dict = cls.get_limits()
        for limit in limit_dict:
            assert isinstance(limit_dict[limit], AwsLimit)
            assert limit == limit_dict[limit].name
            assert limit_dict[limit].service == cls
        assert len(limit_dict) == 3

        trails_limit = limit_dict['Trails Per Region']
        assert trails_limit.limit_type == AWS_TYPE
        assert trails_limit.default_limit == 5

        event_selectors_limit = limit_dict['Event Selectors Per Trail']
        assert event_selectors_limit.limit_type == AWS_TYPE
        assert event_selectors_limit.limit_subtype == \
            'AWS::CloudTrail::EventSelector'
        assert event_selectors_limit.default_limit == 5

        data_resources_limit = limit_dict['Data Resources Per Trail']
        assert data_resources_limit.limit_type == AWS_TYPE
        assert data_resources_limit.limit_subtype == \
            'AWS::CloudTrail::DataResource'
        assert data_resources_limit.default_limit == 250

    def test_get_limits_again(self):
        """Test that existing limits dict is returned on subsequent calls"""
        mock_limits = Mock(spec_set=AwsLimit)
        cls = _CloudTrailService(21, 43, {}, None)
        cls.limits = mock_limits
        response = cls.get_limits()
        assert response == mock_limits

    def test_find_usage(self):
        mock_trails = Mock()
        mock_conf = Mock()
        type(mock_conf).region_name = 'thisregion'
        type(mock_trails)._client_config = mock_conf
        mock_trails.describe_trails.return_value = \
            result_fixtures.CloudTrail.mock_describe_trails

        def se_selectors(*args, **kwargs):
            if kwargs['TrailName'] == 'trailarn2':
                return result_fixtures.CloudTrail.mock_get_event_selectors
            if kwargs['TrailName'] == 'trailarn4':
                raise RuntimeError('foo')
            return {
                'TrailArn': kwargs['TrailName'],
                'EventSelectors': []
            }

        mock_trails.get_event_selectors.side_effect = se_selectors

        with patch('%s.connect' % PATCH_BASE,) as mock_connect:
            cls = _CloudTrailService(21, 43, {}, None)
            cls.conn = mock_trails
            assert cls._have_usage is False
            cls.find_usage()

        assert len(mock_connect.mock_calls) == 1
        assert cls._have_usage is True

        usage = cls.limits['Trails Per Region'].get_current_usage()
        assert len(usage) == 1
        assert usage[0].get_value() == 4

        usage = cls.limits['Event Selectors Per Trail'].get_current_usage()
        assert len(usage) == 2
        assert usage[0].get_value() == 0
        assert usage[1].get_value() == 3

        usage = cls.limits['Data Resources Per Trail'].get_current_usage()
        assert len(usage) == 2
        assert usage[0].get_value() == 0
        assert usage[1].get_value() == 3

    def test_required_iam_permissions(self):
        cls = _CloudTrailService(21, 43, {}, None)
        assert cls.required_iam_permissions() == [
            "cloudtrail:DescribeTrails",
            "cloudtrail:GetEventSelectors",
        ]



"""
awslimitchecker/services/eks.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2018 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

import abc  # noqa
import logging

from .base import _AwsService
from ..limit import AwsLimit
from ..utils import paginate_dict

logger = logging.getLogger(__name__)


class _EksService(_AwsService):

    service_name = 'EKS'
    api_name = 'eks'  # AWS API name to connect to (boto3.client)
    quotas_service_code = 'eks'

    def find_usage(self):
        """
        Determine the current usage for each limit of this service,
        and update corresponding Limit via
        :py:meth:`~.AwsLimit._add_current_usage`.
        """
        logger.debug("Checking usage for service %s", self.service_name)
        self.connect()
        for lim in self.limits.values():
            lim._reset_usage()
        self._find_clusters_usage()
        self._have_usage = True
        logger.debug("Done checking usage.")

    def _find_clusters_usage(self):
        clusters_info = paginate_dict(
            self.conn.list_clusters,
            alc_marker_path=['nextToken'],
            alc_data_path=['clusters'],
            alc_marker_param='nextToken'
        )

        cluster_list = clusters_info['clusters']

        for cluster in cluster_list:
            describe_cluster_response = self.conn.describe_cluster(
                name=cluster
            )
            security_group_id_list = describe_cluster_response['cluster'][
                'resourcesVpcConfig']['securityGroupIds']
            self.limits[
                'Control plane security groups per cluster']._add_current_usage(
                len(security_group_id_list),
                resource_id=cluster,
                aws_type='AWS::EKS::Cluster'
            )
            public_access_cidrs_list = describe_cluster_response['cluster'][
                'resourcesVpcConfig']['publicAccessCidrs']
            self.limits[
                'Public endpoint access CIDR ranges per cluster'
            ]._add_current_usage(
                len(public_access_cidrs_list),
                resource_id=cluster,
                aws_type='AWS::EKS::Cluster'
            )

            list_nodegroup_response = paginate_dict(
                self.conn.list_nodegroups,
                clusterName=cluster,
                alc_marker_path=['nextToken'],
                alc_data_path=['nodegroups'],
                alc_marker_param='nextToken'
            )
            nodegroup_list = list_nodegroup_response['nodegroups']
            self.limits['Managed node groups per cluster']._add_current_usage(
                len(nodegroup_list),
                resource_id=cluster,
                aws_type='AWS::EKS::Cluster')

            list_fargate_profiles_response = paginate_dict(
                self.conn.list_fargate_profiles,
                clusterName=cluster,
                alc_marker_path=['nextToken'],
                alc_data_path=['fargateProfileNames'],
                alc_marker_param='nextToken'
            )
            fargate_profiles_list = list_fargate_profiles_response[
                'fargateProfileNames'
            ]
            self.limits['Fargate profiles per cluster']._add_current_usage(
                len(fargate_profiles_list),
                resource_id=cluster,
                aws_type='AWS::EKS::FargateProfile')

            for fargate_profile_name in fargate_profiles_list:
                fargate_info = self.conn.describe_fargate_profile(
                    clusterName=cluster,
                    fargateProfileName=fargate_profile_name
                )
                profile_selectors = fargate_info['fargateProfile']['selectors']
                self.limits['Selectors per Fargate profile']._add_current_usage(
                    len(profile_selectors),
                    resource_id="{}.{}".format(cluster, fargate_profile_name),
                    aws_type='AWS::EKS::FargateProfile')

                for selector in profile_selectors:
                    label_pairs = selector['labels']
                    self.limits[
                        'Label pairs per Fargate profile selector'
                    ]._add_current_usage(
                        len(label_pairs),
                        resource_id=(
                            "{}.{}.{}".format(
                                cluster,
                                fargate_profile_name,
                                selector
                            )
                        ),
                        aws_type='AWS::EKS::FargateProfile')

        self.limits['Clusters']._add_current_usage(
            len(cluster_list),
            resource_id=self._boto3_connection_kwargs['region_name'],
            aws_type='AWS::EKS::Cluster')

    def get_limits(self):
        """
        Return all known limits for this service, as a dict of their names
        to :py:class:`~.AwsLimit` objects.

        :returns: dict of limit names to :py:class:`~.AwsLimit` objects
        :rtype: dict
        """
        if self.limits != {}:
            return self.limits
        limits = {}
        limits['Clusters'] = AwsLimit(
            'Clusters',
            self,
            100,
            self.warning_threshold,
            self.critical_threshold,
            limit_type='AWS::EKS::Cluster',
        )
        limits['Control plane security groups per cluster'] = AwsLimit(
            'Control plane security groups per cluster',
            self,
            4,
            self.warning_threshold,
            self.critical_threshold,
            limit_type='AWS::EKS::Cluster',
        )
        limits['Managed node groups per cluster'] = AwsLimit(
            'Managed node groups per cluster',
            self,
            30,
            self.warning_threshold,
            self.critical_threshold,
            limit_type='AWS::EKS::Nodegroup',
        )
        limits['Nodes per managed node group'] = AwsLimit(
            'Nodes per managed node group',
            self,
            100,
            self.warning_threshold,
            self.critical_threshold,
            limit_type='AWS::EKS::Cluster',
        )
        limits['Public endpoint access CIDR ranges per cluster'] = AwsLimit(
            'Public endpoint access CIDR ranges per cluster',
            self,
            40,
            self.warning_threshold,
            self.critical_threshold,
            limit_type='AWS::EKS::Cluster',
        )
        limits['Fargate profiles per cluster'] = AwsLimit(
            'Fargate profiles per cluster',
            self,
            10,
            self.warning_threshold,
            self.critical_threshold,
            limit_type='AWS::EKS::FargateProfile',
        )
        limits['Selectors per Fargate profile'] = AwsLimit(
            'Selectors per Fargate profile',
            self,
            5,
            self.warning_threshold,
            self.critical_threshold,
            limit_type='AWS::EKS::FargateProfile',
        )
        limits['Label pairs per Fargate profile selector'] = AwsLimit(
            'Label pairs per Fargate profile selector',
            self,
            5,
            self.warning_threshold,
            self.critical_threshold,
            limit_type='AWS::EKS::FargateProfile',
        )
        self.limits = limits
        return limits

    def required_iam_permissions(self):
        """
        Return a list of IAM Actions required for this Service to function
        properly. All Actions will be shown with an Effect of "Allow"
        and a Resource of "*".

        :returns: list of IAM Action strings
        :rtype: list
        """
        return [
            "eks:ListClusters",
            "eks:DescribeCluster",
            "eks:ListNodegroups",
            "eks:ListFargateProfiles",
            "eks:DescribeFargateProfile",
        ]



"""
awslimitchecker/tests/test_quotas.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

##############################################################################
Copyright 2015-2019 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
##############################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
##############################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
##############################################################################
"""

import sys
from botocore.exceptions import ClientError
import pytest

from awslimitchecker.quotas import ServiceQuotasClient
from awslimitchecker.tests.support import quotas_response

# https://code.google.com/p/mock/issues/detail?id=249
# py>=3.4 should use unittest.mock not the mock package on pypi
if (
        sys.version_info[0] < 3 or
        sys.version_info[0] == 3 and sys.version_info[1] < 4
):
    from mock import patch, call, Mock
else:
    from unittest.mock import patch, call, Mock

pbm = 'awslimitchecker.quotas'
pb = '%s.ServiceQuotasClient' % pbm


class TestConstructor(object):

    def test_init(self):
        cls = ServiceQuotasClient({'foo': 'bar'})
        assert cls._boto3_connection_kwargs == {'foo': 'bar'}
        assert cls._cache == {}
        assert cls.conn is None


class TestQuotasForService(object):

    def setup(self):
        self.cls = ServiceQuotasClient({'foo': 'bar'})

    def test_not_cached(self):
        resp, expected = quotas_response()
        mock_paginator = Mock()
        mock_paginator.paginate.return_value = resp
        mock_conn = Mock()
        mock_conn.get_paginator.return_value = mock_paginator

        def se_connect(cls):
            cls.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_connect
            res = self.cls.quotas_for_service('scode')
        assert res == expected
        assert self.cls._cache == {'scode': expected}
        assert m_connect.mock_calls == [call(self.cls)]
        assert mock_conn.mock_calls == [
            call.get_paginator('list_service_quotas'),
            call.get_paginator().paginate(ServiceCode='scode')
        ]

    def test_cached(self):
        resp, expected = quotas_response()
        mock_paginator = Mock()
        mock_paginator.paginate.return_value = resp
        mock_conn = Mock()
        mock_conn.get_paginator.return_value = mock_paginator

        def se_connect(cls):
            cls.conn = mock_conn

        self.cls._cache = {'scode': expected}

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_connect
            res = self.cls.quotas_for_service('scode')
        assert res == expected
        assert self.cls._cache == {'scode': expected}
        assert m_connect.mock_calls == []
        assert mock_conn.mock_calls == []

    def test_no_such_resource(self):
        mock_paginator = Mock()
        mock_paginator.paginate.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'NoSuchResourceException',
                    'Message': 'The request failed because the specified '
                               'service does not exist.'
                }
            },
            'ListServiceQuotas'
        )
        mock_conn = Mock()
        mock_conn.get_paginator.return_value = mock_paginator

        def se_connect(cls):
            cls.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_connect
            res = self.cls.quotas_for_service('scode')
        assert res == {}
        assert self.cls._cache == {'scode': {}}
        assert m_connect.mock_calls == [call(self.cls)]
        assert mock_conn.mock_calls == [
            call.get_paginator('list_service_quotas'),
            call.get_paginator().paginate(ServiceCode='scode')
        ]

    def test_other_exception(self):
        mock_paginator = Mock()
        mock_paginator.paginate.side_effect = ClientError(
            {
                'Error': {
                    'Code': 'SomeOtherError',
                    'Message': 'My message.'
                }
            },
            'ListServiceQuotas'
        )
        mock_conn = Mock()
        mock_conn.get_paginator.return_value = mock_paginator

        def se_connect(cls):
            cls.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_connect
            with pytest.raises(ClientError):
                self.cls.quotas_for_service('scode')
        assert self.cls._cache == {'scode': {}}
        assert m_connect.mock_calls == [call(self.cls)]
        assert mock_conn.mock_calls == [
            call.get_paginator('list_service_quotas'),
            call.get_paginator().paginate(ServiceCode='scode')
        ]


class TestGetQuotaValue(object):

    def setup(self):
        self.cls = ServiceQuotasClient({'foo': 'bar'})

    def test_happy_path(self):
        self.cls._cache = {
            'scode': {
                'qname': {
                    'QuotaName': 'qname',
                    'QuotaCode': 'qcode',
                    'Value': 12.3,
                    'Unit': 'None'
                }
            }
        }
        res = self.cls.get_quota_value('scode', 'QName')
        assert res == 12.3

    def test_no_quota(self):
        self.cls._cache = {
            'scode': {
                'qname': {
                    'QuotaName': 'qname',
                    'QuotaCode': 'qcode',
                    'Value': 12.3,
                    'Unit': 'None'
                }
            }
        }
        res = self.cls.get_quota_value('scode', 'OtherName')
        assert res is None

    def test_units(self):
        self.cls._cache = {
            'scode': {
                'qname': {
                    'QuotaName': 'qname',
                    'QuotaCode': 'qcode',
                    'Value': 12.3,
                    'Unit': 'Foo'
                }
            }
        }
        res = self.cls.get_quota_value('scode', 'QName')
        assert res is None

    def test_units_converter(self):
        m_conv = Mock()
        m_conv.return_value = 1230.0
        self.cls._cache = {
            'scode': {
                'qname': {
                    'QuotaName': 'qname',
                    'QuotaCode': 'qcode',
                    'Value': 12.3,
                    'Unit': 'Foo'
                }
            }
        }
        res = self.cls.get_quota_value(
            'scode', 'QName', converter=m_conv
        )
        assert res == 1230.0
        assert m_conv.mock_calls == [
            call(12.3, 'Foo', 'None')
        ]



"""
awslimitchecker/tests/services/test_rds.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2018 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

import sys
from awslimitchecker.tests.services import result_fixtures
from awslimitchecker.services.rds import _RDSService

# https://code.google.com/p/mock/issues/detail?id=249
# py>=3.4 should use unittest.mock not the mock package on pypi
if (
        sys.version_info[0] < 3 or
        sys.version_info[0] == 3 and sys.version_info[1] < 4
):
    from mock import patch, call, Mock, DEFAULT
else:
    from unittest.mock import patch, call, Mock, DEFAULT


class Test_RDSService(object):

    pb = 'awslimitchecker.services.rds._RDSService'  # patch base path
    pbm = 'awslimitchecker.services.rds'  # patch base path - module

    def test_init(self):
        """test __init__()"""
        cls = _RDSService(21, 43, {}, None)
        assert cls.service_name == 'RDS'
        assert cls.conn is None
        assert cls.warning_threshold == 21
        assert cls.critical_threshold == 43

    def test_get_limits(self):
        cls = _RDSService(21, 43, {}, None)
        cls.limits = {}
        res = cls.get_limits()
        assert sorted(res.keys()) == sorted([
            # TA  # non-TA / service doc name
            'DB instances',  # 'Instances'
            'Max auths per security group',
            'Storage quota (GB)',  # 'Total storage for all DB instances'
            'DB snapshots per user',  # 'Manual Snapshots'
            'DB security groups',  # 'Security Groups'
            # non-TA
            'Reserved Instances',
            'DB parameter groups',
            'VPC Security Groups',
            'Subnet Groups',
            'Subnets per Subnet Group',
            'Option Groups',
            'Event Subscriptions',
            'Read replicas per master',
            'DB Clusters',
            'DB Cluster Parameter Groups',
            'DB Cluster Roles',
            'DB Instance Roles',
            'Custom Endpoints Per DB Cluster',
            'Manual Cluster Snapshots'
        ])
        for name, limit in res.items():
            assert limit.service == cls
            assert limit.def_warning_threshold == 21
            assert limit.def_critical_threshold == 43

    def test_get_limits_again(self):
        """test that existing limits dict is returned on subsequent calls"""
        mock_limits = Mock()
        cls = _RDSService(21, 43, {}, None)
        cls.limits = mock_limits
        res = cls.get_limits()
        assert res == mock_limits

    def test_find_usage(self):
        mock_conn = Mock()

        with patch('%s.connect' % self.pb) as mock_connect:
            with patch.multiple(
                    self.pb,
                    _find_usage_instances=DEFAULT,
                    _find_usage_subnet_groups=DEFAULT,
                    _find_usage_security_groups=DEFAULT,
                    _update_limits_from_api=DEFAULT,
            ) as mocks:
                cls = _RDSService(21, 43, {}, None)
                cls.conn = mock_conn
                assert cls._have_usage is False
                cls.find_usage()
        assert mock_connect.mock_calls == [call()]
        assert cls._have_usage is True
        for x in [
                '_find_usage_instances',
                '_find_usage_subnet_groups',
                '_find_usage_security_groups',
                '_update_limits_from_api',
        ]:
            assert mocks[x].mock_calls == [call()]

    def test_required_iam_permissions(self):
        cls = _RDSService(21, 43, {}, None)
        assert cls.required_iam_permissions() == [
            "rds:DescribeAccountAttributes",
            "rds:DescribeDBInstances",
            "rds:DescribeDBParameterGroups",
            "rds:DescribeDBSecurityGroups",
            "rds:DescribeDBSnapshots",
            "rds:DescribeDBSubnetGroups",
            "rds:DescribeEventSubscriptions",
            "rds:DescribeOptionGroups",
            "rds:DescribeReservedDBInstances",
        ]

    def test_find_usage_instances(self):
        instances = result_fixtures.RDS.test_find_usage_instances

        mock_conn = Mock()
        mock_paginator = Mock()
        mock_paginator.paginate.return_value = instances
        mock_conn.get_paginator.return_value = mock_paginator

        cls = _RDSService(21, 43, {}, None)
        cls.conn = mock_conn

        cls._find_usage_instances()

        assert mock_conn.mock_calls == [
            call.get_paginator('describe_db_instances'),
            call.get_paginator().paginate()
        ]
        assert mock_paginator.mock_calls == [
            call.paginate()
        ]

        usage = sorted(
            cls.limits['Read replicas per master'].get_current_usage()
        )
        assert len(usage) == 2
        assert usage[0].get_value() == 0
        assert usage[0].resource_id == 'foo'
        assert usage[1].get_value() == 2
        assert usage[1].resource_id == 'baz'

    def test_find_usage_subnet_groups(self):
        data = result_fixtures.RDS.test_find_usage_subnet_groups

        mock_conn = Mock()
        mock_paginator = Mock()
        mock_paginator.paginate.return_value = data
        mock_conn.get_paginator.return_value = mock_paginator

        cls = _RDSService(21, 43, {}, None)
        cls.conn = mock_conn

        cls._find_usage_subnet_groups()

        assert mock_conn.mock_calls == [
            call.get_paginator('describe_db_subnet_groups'),
            call.get_paginator().paginate()
        ]
        assert mock_paginator.mock_calls == [
            call.paginate()
        ]

        usage = sorted(
            cls.limits['Subnets per Subnet Group'].get_current_usage()
        )
        assert len(usage) == 3
        assert usage[0].get_value() == 1
        assert usage[0].aws_type == 'AWS::RDS::DBSubnetGroup'
        assert usage[0].resource_id == "SubnetGroup2"
        assert usage[1].get_value() == 2
        assert usage[1].aws_type == 'AWS::RDS::DBSubnetGroup'
        assert usage[1].resource_id == "SubnetGroup1"
        assert usage[2].get_value() == 3
        assert usage[2].aws_type == 'AWS::RDS::DBSubnetGroup'
        assert usage[2].resource_id == "default"

    def test_find_usage_security_groups(self):
        data = result_fixtures.RDS.test_find_usage_security_groups

        mock_conn = Mock()
        mock_paginator = Mock()
        mock_paginator.paginate.return_value = data
        mock_conn.get_paginator.return_value = mock_paginator

        cls = _RDSService(21, 43, {}, None)
        cls.conn = mock_conn

        cls._find_usage_security_groups()

        assert mock_conn.mock_calls == [
            call.get_paginator('describe_db_security_groups'),
            call.get_paginator().paginate()
        ]
        assert mock_paginator.mock_calls == [
            call.paginate()
        ]

        usage = sorted(cls.limits['VPC Security Groups'].get_current_usage())
        assert len(usage) == 1
        assert usage[0].get_value() == 2
        assert usage[0].aws_type == 'AWS::RDS::DBSecurityGroup'

        usage = sorted(
            cls.limits['Max auths per security group'].get_current_usage()
        )
        assert len(usage) == 5
        assert usage[0].get_value() == 0
        assert usage[0].resource_id == 'MyEmptySecurityGroup'
        assert usage[0].aws_type == 'AWS::RDS::DBSecurityGroup'
        assert usage[1].get_value() == 0
        assert usage[1].resource_id == 'default:vpc-a926c2cc'
        assert usage[1].aws_type == 'AWS::RDS::DBSecurityGroup'
        assert usage[2].get_value() == 1
        assert usage[2].resource_id == 'SecurityGroup1'
        assert usage[2].aws_type == 'AWS::RDS::DBSecurityGroup'
        assert usage[3].get_value() == 2
        assert usage[3].resource_id == 'alctest'
        assert usage[3].aws_type == 'AWS::RDS::DBSecurityGroup'
        assert usage[4].get_value() == 3
        assert usage[4].resource_id == 'SecurityGroup2'
        assert usage[4].aws_type == 'AWS::RDS::DBSecurityGroup'

    def test_update_limits_from_api(self):
        response = result_fixtures.RDS.test_update_limits_from_api

        mock_conn = Mock()
        mock_conn.describe_account_attributes.return_value = response
        with patch('%s.logger' % self.pbm) as mock_logger:
            with patch('%s.connect' % self.pb) as mock_connect:
                cls = _RDSService(21, 43, {}, None)
                cls.conn = mock_conn
                # limits that we still calculate usage for
                cls.limits['Max auths per security group']._add_current_usage(1)
                cls.limits['Subnets per Subnet Group']._add_current_usage(1)
                cls.limits['Read replicas per master']._add_current_usage(1)
                usage_auths = cls.limits[
                    'Max auths per security group'].get_current_usage()
                usage_subnets = cls.limits[
                    'Subnets per Subnet Group'].get_current_usage()
                usage_replicas = cls.limits[
                    'Read replicas per master'].get_current_usage()
                cls._update_limits_from_api()
        assert mock_connect.mock_calls == [call()]
        assert mock_conn.mock_calls == [
            call.describe_account_attributes()
        ]
        assert mock_logger.mock_calls == [
            call.info('Querying RDS DescribeAccountAttributes for limits'),
            call.info(
                'RDS DescribeAccountAttributes returned unknown'
                'limit: %s (max: %s; used: %s)',
                'Foo', 98, 99
            ),
            call.debug('Done setting limits from API.')
        ]

        lim = cls.limits['DB instances']
        assert lim.api_limit == 200
        assert lim.get_current_usage()[0].get_value() == 124

        lim = cls.limits['Reserved Instances']
        assert lim.api_limit == 201
        assert lim.get_current_usage()[0].get_value() == 96

        lim = cls.limits['Storage quota (GB)']
        assert lim.api_limit == 100000
        assert lim.get_current_usage()[0].get_value() == 8320

        lim = cls.limits['DB security groups']
        assert lim.api_limit == 25
        assert lim.get_current_usage()[0].get_value() == 15

        lim = cls.limits['Max auths per security group']
        assert lim.api_limit == 20
        assert lim.get_current_usage() == usage_auths

        lim = cls.limits['DB parameter groups']
        assert lim.api_limit == 50
        assert lim.get_current_usage()[0].get_value() == 39

        lim = cls.limits['DB snapshots per user']
        assert lim.api_limit == 150
        assert lim.get_current_usage()[0].get_value() == 76

        lim = cls.limits['Event Subscriptions']
        assert lim.api_limit == 21
        assert lim.get_current_usage()[0].get_value() == 1

        lim = cls.limits['Subnet Groups']
        assert lim.api_limit == 202
        assert lim.get_current_usage()[0].get_value() == 89

        lim = cls.limits['Option Groups']
        assert lim.api_limit == 22
        assert lim.get_current_usage()[0].get_value() == 2

        lim = cls.limits['Subnets per Subnet Group']
        assert lim.api_limit == 23
        assert lim.get_current_usage() == usage_subnets

        lim = cls.limits['Read replicas per master']
        assert lim.api_limit == 5
        assert lim.get_current_usage() == usage_replicas

        lim = cls.limits['DB Clusters']
        assert lim.api_limit == 40
        assert lim.get_current_usage()[0].get_value() == 3

        lim = cls.limits['DB Cluster Parameter Groups']
        assert lim.api_limit == 51
        assert lim.get_current_usage()[0].get_value() == 6

        lim = cls.limits['Manual Cluster Snapshots']
        assert lim.api_limit == 101
        assert lim.get_current_usage()[0].get_value() == 5

        lim = cls.limits['DB Instance Roles']
        assert lim.api_limit == 11
        assert lim.get_current_usage()[0].get_value() == 1

        lim = cls.limits['DB Cluster Roles']
        assert lim.api_limit == 12
        assert lim.get_current_usage()[0].get_value() == 2

        lim = cls.limits['Custom Endpoints Per DB Cluster']
        assert lim.api_limit == 13
        assert lim.get_current_usage()[0].get_value() == 3



"""
awslimitchecker/services/route53.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2018 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

import abc  # noqa
import logging

from .base import _AwsService
from ..limit import AwsLimit
from ..utils import paginate_dict

logger = logging.getLogger(__name__)


class _Route53Service(_AwsService):
    service_name = 'Route53'
    api_name = 'route53'  # AWS API name to connect to (boto3.client)

    # Route53 limit types
    MAX_RRSETS_BY_ZONE = {
        "type": "MAX_RRSETS_BY_ZONE",
        "name": "Record sets per hosted zone",
        "default_limit": 10000
    }
    MAX_VPCS_ASSOCIATED_BY_ZONE = {
        "type": "MAX_VPCS_ASSOCIATED_BY_ZONE",
        "name": "VPC associations per hosted zone",
        "default_limit": 100
    }

    def find_usage(self):
        """
        Determine the current usage for each limit of this service,
        and update corresponding Limit via
        :py:meth:`~.AwsLimit._add_current_usage`.
        """
        self._have_usage = True

    def _update_limits_from_api(self):
        """
        Query Route53's GetHostedZoneLimit API action, and update limits
        with the quotas returned. Updates ``self.limits``.
        """
        logger.info("Querying Route53 GetHostedZoneLimits for limits")
        self.connect()
        self._find_limit_hosted_zone()
        logger.debug('Done setting limits from API.')

    def get_limits(self):
        """
        Return all known limits for this service, as a dict of their names
        to :py:class:`~.AwsLimit` objects.

        Limits from:
        docs.aws.amazon.com/Route53/latest/DeveloperGuide/DNSLimitations.html

        :returns: dict of limit names to :py:class:`~.AwsLimit` objects
        :rtype: dict
        """
        if not self.limits:
            self.limits = {}
            for item in [self.MAX_RRSETS_BY_ZONE,
                         self.MAX_VPCS_ASSOCIATED_BY_ZONE]:
                self.limits[item["name"]] = AwsLimit(
                    item["name"],
                    self,
                    item["default_limit"],
                    self.warning_threshold,
                    self.critical_threshold,
                    limit_type='AWS::Route53::HostedZone',
                    limit_subtype=item["name"]
                )

        return self.limits

    def _get_hosted_zones(self):
        """
        Return all available hosted zones

        :returns: dict of hosted zones
        :rtype: dict
        """
        results = paginate_dict(
            self.conn.list_hosted_zones,
            alc_marker_path=['NextMarker'],
            alc_data_path=['HostedZones'],
            alc_marker_param='Marker'
        )

        return results["HostedZones"]

    def _get_hosted_zone_limit(self, limit_type, hosted_zone_id):
        """
        Return a hosted zone limit [recordsets|vpc_associations]

        :rtype: dict
        """

        result = self.conn.get_hosted_zone_limit(
            Type=limit_type,
            HostedZoneId=hosted_zone_id
        )

        return result

    def _find_limit_hosted_zone(self):
        """
        Calculate the max recordsets and vpc associations and the current values
        per hosted zone
        """
        for limit_type in [self.MAX_RRSETS_BY_ZONE,
                           self.MAX_VPCS_ASSOCIATED_BY_ZONE]:
            self.limits[limit_type["name"]]._reset_usage()

        for hosted_zone in self._get_hosted_zones():
            for limit_type in [self.MAX_RRSETS_BY_ZONE,
                               self.MAX_VPCS_ASSOCIATED_BY_ZONE]:

                if limit_type == self.MAX_VPCS_ASSOCIATED_BY_ZONE and \
                        not hosted_zone["Config"]["PrivateZone"]:
                    continue

                limit = self._get_hosted_zone_limit(limit_type["type"],
                                                    hosted_zone['Id'])

                self.limits[limit_type["name"]]._add_current_usage(
                    int(limit["Count"]),
                    maximum=int(limit["Limit"]["Value"]),
                    aws_type='AWS::Route53::HostedZone',
                    resource_id=hosted_zone["Name"]
                )

    def required_iam_permissions(self):
        """
        Return a list of IAM Actions required for this Service to function
        properly. All Actions will be shown with an Effect of "Allow"
        and a Resource of "*".

        :returns: list of IAM Action strings
        :rtype: list
        """
        return [
            "route53:GetHostedZone",
            "route53:GetHostedZoneLimit",
            "route53:ListHostedZones"
        ]



"""
awslimitchecker/tests/alerts/test_dummy.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2019 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

from awslimitchecker.alerts.dummy import Dummy


class TestDummy(object):

    def test_init(self):
        cls = Dummy('foo')
        assert cls._region_name == 'foo'

    def test_on_success(self, capsys):
        Dummy('foo').on_success(duration=1.2)
        out, _ = capsys.readouterr()
        assert out == 'awslimitchecker in foo found no problems\n' \
                      'awslimitchecker run duration: 1.2\n'

    def test_on_success_no_duration(self, capsys):
        Dummy('foo').on_success()
        out, _ = capsys.readouterr()
        assert out == 'awslimitchecker in foo found no problems\n'

    def test_on_critical(self, capsys):
        Dummy('foo').on_critical(None, 'probstr', duration=1.2)
        out, _ = capsys.readouterr()
        assert out == 'awslimitchecker in foo found CRITICALS:\n' \
                      'probstr\n' \
                      'awslimitchecker run duration: 1.2\n'

    def test_on_critical_no_duration(self, capsys):
        Dummy('foo').on_critical(None, 'probstr')
        out, _ = capsys.readouterr()
        assert out == 'awslimitchecker in foo found CRITICALS:\n' \
                      'probstr\n'

    def test_on_critical_exception(self, capsys):
        exc = RuntimeError('bar')
        Dummy('foo').on_critical(None, None, exc=exc)
        out, _ = capsys.readouterr()
        assert out == 'awslimitchecker in foo failed with exception: ' \
                      '%s\n' % exc.__repr__()

    def test_on_warning(self, capsys):
        Dummy('foo').on_warning(None, 'probstr', duration=1.2)
        out, _ = capsys.readouterr()
        assert out == 'awslimitchecker in foo found WARNINGS:\n' \
                      'probstr\n' \
                      'awslimitchecker run duration: 1.2\n'

    def test_on_warning_no_duration(self, capsys):
        Dummy('foo').on_warning(None, 'probstr')
        out, _ = capsys.readouterr()
        assert out == 'awslimitchecker in foo found WARNINGS:\n' \
                      'probstr\n'



"""
awslimitchecker/tests/services/test_ec2.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2018 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

import os
import sys
from copy import deepcopy
import pytest
import botocore
from awslimitchecker.tests.services import result_fixtures
from awslimitchecker.services.ec2 import _Ec2Service
from awslimitchecker.limit import AwsLimit
from awslimitchecker.services.ec2 import RI_NO_AZ

# https://code.google.com/p/mock/issues/detail?id=249
# py>=3.4 should use unittest.mock not the mock package on pypi
if (
        sys.version_info[0] < 3 or
        sys.version_info[0] == 3 and sys.version_info[1] < 4
):
    from mock import patch, call, Mock, DEFAULT, PropertyMock
else:
    from unittest.mock import patch, call, Mock, DEFAULT, PropertyMock

fixtures = result_fixtures.EC2()
pb = 'awslimitchecker.services.ec2._Ec2Service'  # patch base path


class TestInit(object):

    def test_simple(self):
        """test __init__()"""
        cls = _Ec2Service(21, 43, {}, None)
        assert cls.service_name == 'EC2'
        assert cls.conn is None
        assert cls.resource_conn is None
        assert cls.warning_threshold == 21
        assert cls.critical_threshold == 43
        assert cls.quotas_service_code == 'ec2'


class TestInstanceTypes(object):

    def test_simple(self):
        cls = _Ec2Service(21, 43, {}, None)
        types = cls._instance_types()
        # NOTE hi1.4xlarge is no longer in the instance type listings,
        # but some accounts might still have a limit for it
        assert len(set(types)) == len(types)
        assert len(types) == 268
        assert 't2.micro' in types
        assert 'r3.8xlarge' in types
        assert 'c3.large' in types
        assert 'i2.4xlarge' in types
        assert 'i3.16xlarge' in types
        assert 'd2.2xlarge' in types
        assert 'g2.8xlarge' in types
        assert 'hs1.8xlarge' in types
        assert 'cg1.4xlarge' in types
        assert 'm4.4xlarge' in types
        assert 'p2.16xlarge' in types
        assert 'm4.16xlarge' in types
        assert 'x1.32xlarge' in types
        assert 'z1d.12xlarge' in types
        assert 'u-24tb1.metal' in types
        assert 'm5n.metal' in types


class TestGetLimits(object):

    def test_nonvcpu(self):
        cls = _Ec2Service(21, 43, {}, None)
        cls.limits = {}
        with patch.multiple(
            pb,
            _get_limits_instances_nonvcpu=DEFAULT,
            _get_limits_instances_vcpu=DEFAULT,
            _get_limits_networking=DEFAULT,
            _get_limits_spot=DEFAULT,
            autospec=True
        ) as mocks:
            mocks['_get_limits_instances_nonvcpu'].return_value = {
                'ec2lname': 'ec2lval'
            }
            mocks['_get_limits_instances_vcpu'].return_value = {
                'fooname': 'fooval'
            }
            mocks['_get_limits_networking'].return_value = {'vpck': 'vpcv'}
            mocks['_get_limits_spot'].return_value = {'spotk': 'spotv'}
            with patch(
                '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
            ) as m_use_vcpu:
                m_use_vcpu.return_value = False
                res = cls.get_limits()
        assert res == {
            'ec2lname': 'ec2lval',
            'spotk': 'spotv',
            'vpck': 'vpcv',
        }
        assert mocks['_get_limits_instances_nonvcpu'].mock_calls == [
            call(cls)
        ]
        assert mocks['_get_limits_instances_vcpu'].mock_calls == []
        assert mocks['_get_limits_networking'].mock_calls == [call(cls)]
        assert mocks['_get_limits_spot'].mock_calls == [call(cls)]

    def test_vcpu(self):
        cls = _Ec2Service(21, 43, {}, None)
        cls.limits = {}
        with patch.multiple(
            pb,
            _get_limits_instances_nonvcpu=DEFAULT,
            _get_limits_instances_vcpu=DEFAULT,
            _get_limits_networking=DEFAULT,
            _get_limits_spot=DEFAULT,
            autospec=True
        ) as mocks:
            mocks['_get_limits_instances_nonvcpu'].return_value = {
                'ec2lname': 'ec2lval'
            }
            mocks['_get_limits_instances_vcpu'].return_value = {
                'fooname': 'fooval'
            }
            mocks['_get_limits_networking'].return_value = {'vpck': 'vpcv'}
            mocks['_get_limits_spot'].return_value = {'spotk': 'spotv'}
            with patch(
                '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
            ) as m_use_vcpu:
                m_use_vcpu.return_value = True
                res = cls.get_limits()
        assert res == {
            'fooname': 'fooval',
            'spotk': 'spotv',
            'vpck': 'vpcv',
        }
        assert mocks['_get_limits_instances_nonvcpu'].mock_calls == []
        assert mocks['_get_limits_instances_vcpu'].mock_calls == [call(cls)]
        assert mocks['_get_limits_networking'].mock_calls == [call(cls)]
        assert mocks['_get_limits_spot'].mock_calls == [call(cls)]

    def test_get_again(self):
        """test that existing limits dict is returned on subsequent calls"""
        cls = _Ec2Service(21, 43, {}, None)
        cls.limits = {'foo': 'bar'}
        with patch.multiple(
            pb,
            _get_limits_instances_nonvcpu=DEFAULT,
            _get_limits_instances_vcpu=DEFAULT,
            _get_limits_networking=DEFAULT,
            _get_limits_spot=DEFAULT,
            autospec=True
        ) as mocks:
            mocks['_get_limits_instances_nonvcpu'].return_value = {
                'ec2lname': 'ec2lval'
            }
            mocks['_get_limits_instances_vcpu'].return_value = {
                'fooname': 'fooval'
            }
            mocks['_get_limits_networking'].return_value = {'vpck': 'vpcv'}
            mocks['_get_limits_spot'].return_value = {'spotk': 'spotv'}
            with patch(
                '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
            ) as m_use_vcpu:
                m_use_vcpu.return_value = False
                res = cls.get_limits()
        assert res == {'foo': 'bar'}
        assert mocks['_get_limits_instances_nonvcpu'].mock_calls == []
        assert mocks['_get_limits_instances_vcpu'].mock_calls == []
        assert mocks['_get_limits_networking'].mock_calls == []
        assert mocks['_get_limits_spot'].mock_calls == []

    def test_all_nonvcpu(self):
        """test some things all limits should conform to"""
        with patch(
                '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
        ) as m_use_vcpu:
            m_use_vcpu.return_value = False
            cls = _Ec2Service(21, 43, {}, None)
            limits = cls.get_limits()
        for x in limits:
            assert isinstance(limits[x], AwsLimit)
            assert x == limits[x].name
            assert limits[x].service == cls

    def test_all_vcpu(self):
        """test some things all limits should conform to"""
        with patch(
                '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
        ) as m_use_vcpu:
            m_use_vcpu.return_value = True
            cls = _Ec2Service(21, 43, {}, None)
            limits = cls.get_limits()
        for x in limits:
            assert isinstance(limits[x], AwsLimit)
            assert x == limits[x].name
            assert limits[x].service == cls


class TestGetLimitsInstancesNonvcpu(object):

    def test_simple(self):
        cls = _Ec2Service(21, 43, {}, None)
        limits = cls._get_limits_instances_nonvcpu()
        assert len(limits) == 269
        # check a random subset of limits
        t2_micro = limits['Running On-Demand t2.micro instances']
        assert t2_micro.default_limit == 20
        assert t2_micro.limit_type == 'On-Demand instances'
        assert t2_micro.limit_subtype == 't2.micro'
        c4_8xlarge = limits['Running On-Demand c4.8xlarge instances']
        assert c4_8xlarge.default_limit == 5
        assert c4_8xlarge.limit_type == 'On-Demand instances'
        assert c4_8xlarge.limit_subtype == 'c4.8xlarge'
        i2_8xlarge = limits['Running On-Demand i2.8xlarge instances']
        assert i2_8xlarge.default_limit == 2
        assert i2_8xlarge.limit_type == 'On-Demand instances'
        assert i2_8xlarge.limit_subtype == 'i2.8xlarge'
        i3_16xlarge = limits['Running On-Demand i3.16xlarge instances']
        assert i3_16xlarge.default_limit == 2
        assert i3_16xlarge.limit_type == 'On-Demand instances'
        assert i3_16xlarge.limit_subtype == 'i3.16xlarge'
        m4_16xlarge = limits['Running On-Demand m4.16xlarge instances']
        assert m4_16xlarge.default_limit == 5
        assert m4_16xlarge.limit_type == 'On-Demand instances'
        assert m4_16xlarge.limit_subtype == 'm4.16xlarge'
        p2_16xlarge = limits['Running On-Demand p2.16xlarge instances']
        assert p2_16xlarge.default_limit == 1
        assert p2_16xlarge.limit_type == 'On-Demand instances'
        assert p2_16xlarge.limit_subtype == 'p2.16xlarge'
        all_ec2 = limits['Running On-Demand EC2 instances']
        assert all_ec2.default_limit == 20
        assert all_ec2.limit_type == 'On-Demand instances'
        assert all_ec2.limit_subtype is None
        assert 'Running On-Demand m4.4xlarge instances' in limits
        for lname, lim in limits.items():
            assert lim.limit_type == 'On-Demand instances'
            itype = lim.limit_subtype
            if itype is not None:
                assert lname == 'Running On-Demand %s instances' % itype
                assert lim.ta_limit_name == 'On-Demand instances - %s' % itype


class TestGetLimitsInstancesVcpu(object):

    def test_simple(self):
        cls = _Ec2Service(21, 43, {}, None)
        limits = cls._get_limits_instances_vcpu()
        assert len(limits) == 5
        for k in ['f', 'g', 'p', 'x']:
            lim = limits['Running On-Demand All %s instances' % k.upper()]
            assert lim.default_limit == 128
            assert lim.limit_type == 'On-Demand instances'
            assert lim.limit_subtype == k.upper()
        k = 'Running On-Demand All Standard ' \
            '(A, C, D, H, I, M, R, T, Z) instances'
        lim = limits[k]
        assert lim.default_limit == 1152
        assert lim.limit_type == 'On-Demand instances'
        assert lim.limit_subtype == 'Standard'


class TestFindUsage(object):

    def test_nonvcpu(self):
        with patch.multiple(
                pb,
                connect=DEFAULT,
                _find_usage_instances_nonvcpu=DEFAULT,
                _find_usage_instances_vcpu=DEFAULT,
                _find_usage_networking_sgs=DEFAULT,
                _find_usage_networking_eips=DEFAULT,
                _find_usage_networking_eni_sg=DEFAULT,
                _find_usage_spot_instances=DEFAULT,
                _find_usage_spot_fleets=DEFAULT,
                autospec=True,
        ) as mocks:
            with patch(
                    '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
            ) as m_use_vcpu:
                m_use_vcpu.return_value = False
                cls = _Ec2Service(21, 43, {}, None)
                assert cls._have_usage is False
                cls.find_usage()
        assert cls._have_usage is True
        assert mocks['_find_usage_instances_nonvcpu'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_instances_vcpu'].mock_calls == []
        assert mocks['_find_usage_networking_sgs'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_networking_eips'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_networking_eni_sg'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_spot_instances'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_spot_fleets'].mock_calls == [
            call(cls)
        ]

    def test_vcpu(self):
        with patch.multiple(
                pb,
                connect=DEFAULT,
                _find_usage_instances_nonvcpu=DEFAULT,
                _find_usage_instances_vcpu=DEFAULT,
                _find_usage_networking_sgs=DEFAULT,
                _find_usage_networking_eips=DEFAULT,
                _find_usage_networking_eni_sg=DEFAULT,
                _find_usage_spot_instances=DEFAULT,
                _find_usage_spot_fleets=DEFAULT,
                autospec=True,
        ) as mocks:
            with patch(
                    '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
            ) as m_use_vcpu:
                m_use_vcpu.return_value = True
                cls = _Ec2Service(21, 43, {}, None)
                assert cls._have_usage is False
                cls.find_usage()
        assert cls._have_usage is True
        assert mocks['_find_usage_instances_nonvcpu'].mock_calls == []
        assert mocks['_find_usage_instances_vcpu'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_networking_sgs'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_networking_eips'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_networking_eni_sg'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_spot_instances'].mock_calls == [
            call(cls)
        ]
        assert mocks['_find_usage_spot_fleets'].mock_calls == [
            call(cls)
        ]


class TestInstanceUsage(object):

    def test_simple(self):
        mock_t2_micro = Mock(spec_set=AwsLimit)
        mock_r3_2xlarge = Mock(spec_set=AwsLimit)
        mock_c4_4xlarge = Mock(spec_set=AwsLimit)
        mock_m4_8xlarge = Mock(spec_set=AwsLimit)
        limits = {
            'Running On-Demand t2.micro instances': mock_t2_micro,
            'Running On-Demand r3.2xlarge instances': mock_r3_2xlarge,
            'Running On-Demand c4.4xlarge instances': mock_c4_4xlarge,
            'Running On-Demand m4.8xlarge instances': mock_m4_8xlarge,
        }

        cls = _Ec2Service(21, 43, {}, None)
        mock_conn = Mock()

        retval = fixtures.test_instance_usage
        mock_conn.instances.all.return_value = retval

        cls.resource_conn = mock_conn
        cls.limits = limits

        with patch('awslimitchecker.services.ec2._Ec2Service._instance_types',
                   autospec=True) as mock_itypes:
            mock_itypes.return_value = [
                't2.micro',
                'r3.2xlarge',
                'c4.4xlarge',
                'm4.8xlarge',
            ]
            res = cls._instance_usage()
        assert res == {
            'az1a': {
                't2.micro': 1,
                'r3.2xlarge': 1,
                'c4.4xlarge': 1,
                'm4.8xlarge': 1,
            }
        }
        assert mock_conn.mock_calls == [
            call.instances.all()
        ]

    def test_key_error(self):
        mock_conn = Mock()
        data = fixtures.test_instance_usage_key_error
        mock_conn.instances.all.return_value = data
        cls = _Ec2Service(21, 43, {}, None)
        cls.resource_conn = mock_conn
        cls.limits = {'Running On-Demand t2.micro instances': Mock()}

        with patch(
                '%s._instance_types' % pb,
                autospec=True) as mock_itypes:
            with patch('awslimitchecker.services.ec2.logger') as mock_logger:
                mock_itypes.return_value = ['t2.micro']
                cls._instance_usage()
        assert mock_logger.mock_calls == [
            call.debug('Getting usage for on-demand instances'),
            call.error("ERROR - unknown instance type '%s'; not counting",
                       'foobar'),
        ]
        assert mock_conn.mock_calls == [
            call.instances.all()
        ]


class TestInstanceUsageVcpu(object):

    def test_no_RIs(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_conn = Mock()
        retval = fixtures.test_instance_usage_vcpu
        mock_conn.instances.all.return_value = retval
        cls.resource_conn = mock_conn

        res = cls._instance_usage_vcpu({})
        assert res == {
            'c': 16,
            'f': 72,
            'g': 48,
            'm': 32,
            'r': 16,
            't': 2,
            'p': 128,
            'x': 256,
        }
        assert mock_conn.mock_calls == [
            call.instances.all()
        ]

    def test_with_RIs(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_conn = Mock()
        retval = fixtures.test_instance_usage_vcpu
        mock_conn.instances.all.return_value = retval
        cls.resource_conn = mock_conn

        res = cls._instance_usage_vcpu({
            'az1a': {
                'f1.2xlarge': 10,
                'c4.4xlarge': 8,
                'c4.2xlarge': 16,
            },
            'az1c': {
                'x1e.32xlarge': 1,
                'p2.8xlarge': 1,
                'p2.16xlarge': 1
            }
        })
        assert res == {
            'f': 64,
            'g': 48,
            'm': 32,
            'r': 16,
            't': 2,
            'p': 32,
            'x': 128,
        }
        assert mock_conn.mock_calls == [
            call.instances.all()
        ]


class TestGetReservedInstanceCount(object):

    def test_simple(self):
        response = fixtures.test_get_reserved_instance_count

        cls = _Ec2Service(21, 43, {}, None)
        mock_client_conn = Mock()
        cls.conn = mock_client_conn
        mock_client_conn.describe_reserved_instances.return_value = response
        mock_conn = Mock()
        cls.resource_conn = mock_conn

        res = cls._get_reserved_instance_count()
        assert res == {
            'az1': {
                'it1': 10,
            },
            'az2': {
                'it2': 98,
            },
            RI_NO_AZ: {
                'it2': 9,
                'it3': 6
            }
        }
        assert mock_conn.mock_calls == []
        assert mock_client_conn.mock_calls == [
            call.describe_reserved_instances()
        ]


class TestFindUsageInstancesNonvcpu(object):

    def test_simple(self):
        iusage = {
            'us-east-1': {
                't2.micro': 2,
                'r3.2xlarge': 10,
                'c4.4xlarge': 3,
                'c4.large': 2,
            },
            'fooaz': {
                't2.micro': 32,
                'c4.large': 2,
            },
            'us-west-1': {
                't2.micro': 5,
                'r3.2xlarge': 5,
                'c4.4xlarge': 2,
            },
        }

        ri_count = {
            'us-east-1': {
                't2.micro': 10,
                'r3.2xlarge': 2,
            },
            'us-west-1': {
                't2.micro': 1,
                'r3.2xlarge': 5,
            },
            RI_NO_AZ: {
                't2.micro': 1,
                'c4.large': 50,
            }
        }

        mock_t2_micro = Mock(spec_set=AwsLimit)
        mock_r3_2xlarge = Mock(spec_set=AwsLimit)
        mock_c4_4xlarge = Mock(spec_set=AwsLimit)
        mock_c4_large = Mock(spec_set=AwsLimit)
        mock_all_ec2 = Mock(spec_set=AwsLimit)
        limits = {
            'Running On-Demand t2.micro instances': mock_t2_micro,
            'Running On-Demand r3.2xlarge instances': mock_r3_2xlarge,
            'Running On-Demand c4.4xlarge instances': mock_c4_4xlarge,
            'Running On-Demand c4.large instances': mock_c4_large,
            'Running On-Demand EC2 instances': mock_all_ec2,
        }

        cls = _Ec2Service(21, 43, {}, None)
        mock_conn = Mock()
        cls.resource_conn = mock_conn
        cls.limits = limits
        with patch('%s._instance_usage' % pb,
                   autospec=True) as mock_inst_usage:
            with patch('%s._get_reserved_instance_count' % pb,
                       autospec=True) as mock_res_inst_count:
                mock_inst_usage.return_value = iusage
                mock_res_inst_count.return_value = ri_count
                cls._find_usage_instances_nonvcpu()
        assert mock_t2_micro.mock_calls == [call._add_current_usage(
            36,
            aws_type='AWS::EC2::Instance'
        )]
        assert mock_r3_2xlarge.mock_calls == [call._add_current_usage(
            8,
            aws_type='AWS::EC2::Instance'
        )]
        assert mock_c4_4xlarge.mock_calls == [call._add_current_usage(
            5,
            aws_type='AWS::EC2::Instance'
        )]
        assert mock_c4_large.mock_calls == [call._add_current_usage(
            4,
            aws_type='AWS::EC2::Instance'
        )]
        assert mock_all_ec2.mock_calls == [call._add_current_usage(
            53,
            aws_type='AWS::EC2::Instance'
        )]
        assert mock_inst_usage.mock_calls == [call(cls)]
        assert mock_res_inst_count.mock_calls == [call(cls)]
        assert mock_conn.mock_calls == []


class TestFindUsageInstancesVcpu(object):

    def test_happy_path(self):
        usage = {
            'c': 16,
            'f': 72,
            'g': 48,
            'm': 32,
            'r': 16,
            't': 2,
            'p': 128,
            'x': 256,
            'a': 512,
            'k': 3
        }

        mock_f = Mock(spec_set=AwsLimit)
        mock_g = Mock(spec_set=AwsLimit)
        mock_p = Mock(spec_set=AwsLimit)
        mock_x = Mock(spec_set=AwsLimit)
        mock_std = Mock(spec_set=AwsLimit)
        limits = {
            'Running On-Demand All F instances': mock_f,
            'Running On-Demand All G instances': mock_g,
            'Running On-Demand All P instances': mock_p,
            'Running On-Demand All X instances': mock_x,
            'Running On-Demand All Standard '
            '(A, C, D, H, I, M, R, T, Z) instances': mock_std
        }

        cls = _Ec2Service(21, 43, {}, None)
        cls.limits = limits

        with patch(
            '%s._get_reserved_instance_count' % pb, autospec=True
        ) as m_gric:
            with patch('%s._instance_usage_vcpu' % pb, autospec=True) as m_iuv:
                m_gric.return_value = {'res': 'inst'}
                m_iuv.return_value = usage
                cls._find_usage_instances_vcpu()
        assert m_gric.mock_calls == [call(cls)]
        assert m_iuv.mock_calls == [call(cls, {'res': 'inst'})]
        assert mock_f.mock_calls == [
            call._add_current_usage(72, aws_type='AWS::EC2::Instance')
        ]
        assert mock_g.mock_calls == [
            call._add_current_usage(48, aws_type='AWS::EC2::Instance')
        ]
        assert mock_p.mock_calls == [
            call._add_current_usage(128, aws_type='AWS::EC2::Instance')
        ]
        assert mock_x.mock_calls == [
            call._add_current_usage(256, aws_type='AWS::EC2::Instance')
        ]
        assert mock_std.mock_calls == [
            call._add_current_usage(581, aws_type='AWS::EC2::Instance')
        ]

    def test_default_zero(self):
        usage = {
            'c': 16,
            'm': 32,
            'r': 16,
            't': 2,
            'p': 128,
            'x': 256,
            'a': 512,
            'k': 3
        }

        mock_f = Mock(spec_set=AwsLimit)
        mock_g = Mock(spec_set=AwsLimit)
        mock_p = Mock(spec_set=AwsLimit)
        mock_x = Mock(spec_set=AwsLimit)
        mock_std = Mock(spec_set=AwsLimit)
        limits = {
            'Running On-Demand All F instances': mock_f,
            'Running On-Demand All G instances': mock_g,
            'Running On-Demand All P instances': mock_p,
            'Running On-Demand All X instances': mock_x,
            'Running On-Demand All Standard '
            '(A, C, D, H, I, M, R, T, Z) instances': mock_std
        }

        cls = _Ec2Service(21, 43, {}, None)
        cls.limits = limits

        with patch(
            '%s._get_reserved_instance_count' % pb, autospec=True
        ) as m_gric:
            with patch('%s._instance_usage_vcpu' % pb, autospec=True) as m_iuv:
                m_gric.return_value = {'res': 'inst'}
                m_iuv.return_value = usage
                cls._find_usage_instances_vcpu()
        assert m_gric.mock_calls == [call(cls)]
        assert m_iuv.mock_calls == [call(cls, {'res': 'inst'})]
        assert mock_f.mock_calls == [
            call._add_current_usage(0, aws_type='AWS::EC2::Instance')
        ]
        assert mock_g.mock_calls == [
            call._add_current_usage(0, aws_type='AWS::EC2::Instance')
        ]
        assert mock_p.mock_calls == [
            call._add_current_usage(128, aws_type='AWS::EC2::Instance')
        ]
        assert mock_x.mock_calls == [
            call._add_current_usage(256, aws_type='AWS::EC2::Instance')
        ]
        assert mock_std.mock_calls == [
            call._add_current_usage(581, aws_type='AWS::EC2::Instance')
        ]


class TestRequiredIamPermissions(object):

    def test_simple(self):
        cls = _Ec2Service(21, 43, {}, None)
        assert len(cls.required_iam_permissions()) == 19
        assert cls.required_iam_permissions() == [
            "ec2:DescribeAccountAttributes",
            "ec2:DescribeAddresses",
            "ec2:DescribeInstances",
            "ec2:DescribeInternetGateways",
            "ec2:DescribeNetworkAcls",
            "ec2:DescribeNetworkInterfaces",
            "ec2:DescribeReservedInstances",
            "ec2:DescribeRouteTables",
            "ec2:DescribeSecurityGroups",
            "ec2:DescribeSnapshots",
            "ec2:DescribeSpotDatafeedSubscription",
            "ec2:DescribeSpotFleetInstances",
            "ec2:DescribeSpotFleetRequestHistory",
            "ec2:DescribeSpotFleetRequests",
            "ec2:DescribeSpotInstanceRequests",
            "ec2:DescribeSpotPriceHistory",
            "ec2:DescribeSubnets",
            "ec2:DescribeVolumes",
            "ec2:DescribeVpcs",
        ]


class TestFindUsageNetworkingSgs(object):

    def test_simple(self):
        mocks = fixtures.test_find_usage_networking_sgs

        mock_conn = Mock()
        mock_conn.security_groups.all.return_value = mocks

        cls = _Ec2Service(21, 43, {}, None)
        cls.resource_conn = mock_conn

        with patch('awslimitchecker.services.ec2.logger') as mock_logger:
            cls._find_usage_networking_sgs()
        assert mock_logger.mock_calls == [
            call.debug("Getting usage for EC2 VPC resources"),
        ]
        limit = cls.limits['VPC security groups per Region']
        # relies on AwsLimitUsage sorting by numeric usage value
        sorted_usage = sorted(limit.get_current_usage())
        assert len(sorted_usage) == 1
        assert sorted_usage[0].limit == limit
        assert sorted_usage[0].get_value() == 3
        assert sorted_usage[0].aws_type == 'AWS::EC2::SecurityGroup'

        limit = cls.limits['Rules per VPC security group']
        sorted_usage = sorted(limit.get_current_usage())
        assert len(sorted_usage) == 3
        assert sorted_usage[0].limit == limit
        assert sorted_usage[0].resource_id == 'sg-1'
        assert sorted_usage[0].get_value() == 0
        assert sorted_usage[1].limit == limit
        assert sorted_usage[1].resource_id == 'sg-2'
        # ingress: IPv4 = 15; IPv6 = 13
        # egress: IPv4 = 5; IPv6 = 7
        assert sorted_usage[1].get_value() == 15
        assert sorted_usage[2].limit == limit
        assert sorted_usage[2].resource_id == 'sg-3'
        # ingress: IPv4 = 4; IPv6 = 5
        # egress: IPv4 = 22; IPv6 = 29
        assert sorted_usage[2].get_value() == 29
        assert mock_conn.mock_calls == [
            call.security_groups.all()
        ]


class TestFindUsageNetworkingEips(object):

    def test_simple(self):
        mocks = fixtures.test_find_usage_networking_eips

        mock_conn = Mock()
        mock_conn.classic_addresses.all.return_value = mocks['Classic']
        mock_conn.vpc_addresses.all.return_value = mocks['Vpc']
        cls = _Ec2Service(21, 43, {}, None)
        cls.resource_conn = mock_conn

        with patch('awslimitchecker.services.ec2.logger') as mock_logger:
            cls._find_usage_networking_eips()
        assert mock_logger.mock_calls == [
            call.debug("Getting usage for EC2 EIPs"),
        ]
        limit = cls.limits['VPC Elastic IP addresses (EIPs)']
        usage = limit.get_current_usage()
        assert len(usage) == 1
        assert usage[0].limit == limit
        assert usage[0].get_value() == 2
        assert usage[0].resource_id is None
        assert usage[0].aws_type == 'AWS::EC2::EIP'

        limit = cls.limits['Elastic IP addresses (EIPs)']
        usage = limit.get_current_usage()
        assert len(usage) == 1
        assert usage[0].limit == limit
        assert usage[0].get_value() == 1
        assert usage[0].resource_id is None
        assert usage[0].aws_type == 'AWS::EC2::EIP'

        assert mock_conn.mock_calls == [
            call.vpc_addresses.all(),
            call.classic_addresses.all()
        ]


class TestFindUsageNetworkingEniSg(object):

    def test_simple(self):
        mocks = fixtures.test_find_usage_networking_eni_sg

        mock_conn = Mock()
        mock_conn.network_interfaces.all.return_value = mocks
        cls = _Ec2Service(21, 43, {}, None)
        cls.resource_conn = mock_conn
        with patch('awslimitchecker.services.ec2.logger') as mock_logger:
            cls._find_usage_networking_eni_sg()
        assert mock_logger.mock_calls == [
            call.debug("Getting usage for EC2 Network Interfaces"),
        ]
        limit = cls.limits['VPC security groups per elastic network interface']
        sorted_usage = sorted(limit.get_current_usage())
        assert len(sorted_usage) == 3
        assert sorted_usage[0].limit == limit
        assert sorted_usage[0].resource_id == 'if-1'
        assert sorted_usage[0].get_value() == 0
        assert sorted_usage[1].limit == limit
        assert sorted_usage[1].resource_id == 'if-2'
        assert sorted_usage[1].get_value() == 3
        assert sorted_usage[2].limit == limit
        assert sorted_usage[2].resource_id == 'if-3'
        assert sorted_usage[2].get_value() == 8
        assert mock_conn.mock_calls == [
            call.network_interfaces.all()
        ]


class TestGetLimitsNetworking(object):

    def test_simple(self):
        cls = _Ec2Service(21, 43, {}, None)
        limits = cls._get_limits_networking()
        expected = [
            'VPC security groups per Region',
            'Rules per VPC security group',
            'VPC Elastic IP addresses (EIPs)',
            'Elastic IP addresses (EIPs)',
            'VPC security groups per elastic network interface',
        ]
        assert sorted(limits.keys()) == sorted(expected)
        assert limits[
            'VPC Elastic IP addresses (EIPs)'].ta_service_name == 'VPC'
        assert limits[
            'VPC Elastic IP addresses (EIPs)'
        ].quotas_service_code == 'ec2'
        assert limits[
            'VPC Elastic IP addresses (EIPs)'
        ].quota_name == 'Number of EIPs - VPC EIPs'
        assert limits[
            'VPC Elastic IP addresses (EIPs)'
        ].quotas_unit == 'None'
        assert limits[
            'Elastic IP addresses (EIPs)'
        ].quotas_service_code == 'ec2'
        assert limits[
            'Elastic IP addresses (EIPs)'
        ].quota_name == 'Elastic IP addresses for EC2-Classic'
        assert limits[
            'Elastic IP addresses (EIPs)'
        ].quotas_unit == 'None'
        vpcsg = limits['VPC security groups per Region']
        assert vpcsg.quota_name == 'VPC security groups per Region'
        assert vpcsg.quotas_service_code == 'vpc'
        assert vpcsg.default_limit == 2500


class TestGetLimitsSpot(object):

    def test_simple(self):
        cls = _Ec2Service(21, 43, {}, None)
        limits = cls._get_limits_spot()
        expected = [
            'Max spot instance requests per region',
            'Max active spot fleets per region',
            'Max launch specifications per spot fleet',
            'Max target capacity per spot fleet',
            'Max target capacity for all spot fleets in region'
        ]
        assert sorted(limits.keys()) == sorted(expected)


class TestFindUsageSpotInstances(object):

    def test_happy_path(self):
        data = fixtures.test_find_usage_spot_instances
        mock_conn = Mock()
        mock_client_conn = Mock()
        mock_client_conn.describe_spot_instance_requests.return_value = data
        cls = _Ec2Service(21, 43, {}, None)
        cls.resource_conn = mock_conn
        cls.conn = mock_client_conn
        with patch('awslimitchecker.services.ec2.logger') as mock_logger:
            cls._find_usage_spot_instances()
        assert mock_conn.mock_calls == []
        assert mock_client_conn.mock_calls == [
            call.describe_spot_instance_requests()
        ]
        lim = cls.limits['Max spot instance requests per region']
        usage = lim.get_current_usage()
        assert len(usage) == 1
        assert usage[0].get_value() == 2
        assert mock_logger.mock_calls == [
            call.debug('Getting spot instance request usage'),
            call.debug('NOT counting spot instance request %s state=%s',
                       'reqID1', 'closed'),
            call.debug('Counting spot instance request %s state=%s',
                       'reqID2', 'active'),
            call.debug('Counting spot instance request %s state=%s',
                       'reqID3', 'open'),
            call.debug('NOT counting spot instance request %s state=%s',
                       'reqID4', 'failed')
        ]

    def test_unsupported(self):
        mock_client_conn = Mock()
        err = botocore.exceptions.ClientError(
            {'Error': {'Code': 'UnsupportedOperation'}},
            'operation',
        )
        mock_client_conn.describe_spot_instance_requests.side_effect = err
        cls = _Ec2Service(21, 43, {}, None)
        cls.conn = mock_client_conn
        cls._find_usage_spot_instances()
        lim = cls.limits['Max spot instance requests per region']
        usage = lim.get_current_usage()
        assert len(usage) == 0

    def test_unknown_code(self):
        mock_client_conn = Mock()
        err = botocore.exceptions.ClientError(
            {'Error': {'Code': 'SomeCode'}},
            'operation',
        )
        mock_client_conn.describe_spot_instance_requests.side_effect = err
        cls = _Ec2Service(21, 43, {}, None)
        cls.conn = mock_client_conn
        with pytest.raises(botocore.exceptions.ClientError):
            cls._find_usage_spot_instances()

    def test_unknown_error(self):
        mock_client_conn = Mock()
        err = RuntimeError
        mock_client_conn.describe_spot_instance_requests.side_effect = err
        cls = _Ec2Service(21, 43, {}, None)
        cls.conn = mock_client_conn
        with pytest.raises(RuntimeError):
            cls._find_usage_spot_instances()


class TestFindUsageSpotFleets(object):

    def test_simple(self):
        data = fixtures.test_find_usage_spot_fleets
        mock_conn = Mock()
        mock_client_conn = Mock()
        mock_client_conn.describe_spot_fleet_requests.return_value = data
        cls = _Ec2Service(21, 43, {}, None)
        cls.resource_conn = mock_conn
        cls.conn = mock_client_conn
        with patch('awslimitchecker.services.ec2.logger') as mock_logger:
            cls._find_usage_spot_fleets()
        assert mock_conn.mock_calls == []
        assert mock_client_conn.mock_calls == [
            call.describe_spot_fleet_requests()
        ]

        total = cls.limits['Max active spot fleets per '
                           'region'].get_current_usage()
        assert len(total) == 1
        assert total[0].get_value() == 2

        totalcap = cls.limits['Max target capacity for all spot fleets '
                              'in region'].get_current_usage()
        assert len(totalcap) == 1
        assert totalcap[0].get_value() == 44

        cap_per_fleet = cls.limits['Max target capacity per spot '
                                   'fleet'].get_current_usage()
        assert len(cap_per_fleet) == 2
        assert cap_per_fleet[0].get_value() == 11
        assert cap_per_fleet[0].resource_id == 'req2'
        assert cap_per_fleet[1].get_value() == 33
        assert cap_per_fleet[1].resource_id == 'req4'

        launch_specs = cls.limits['Max launch specifications '
                                  'per spot fleet'].get_current_usage()
        assert len(launch_specs) == 2
        assert launch_specs[0].get_value() == 3
        assert launch_specs[0].resource_id == 'req2'
        assert launch_specs[1].get_value() == 1
        assert launch_specs[1].resource_id == 'req4'

        assert mock_logger.mock_calls == [
            call.debug('Getting spot fleet request usage'),
            call.debug('Skipping spot fleet request %s in state %s', 'req1',
                       'failed'),
            call.debug('Skipping spot fleet request %s in state %s',
                       'req3', 'modifying')
        ]

    def test_paginated(self):
        data = deepcopy(fixtures.test_find_usage_spot_fleets)
        data['NextToken'] = 'string'
        mock_conn = Mock()
        mock_client_conn = Mock()
        mock_client_conn.describe_spot_fleet_requests.return_value = data
        cls = _Ec2Service(21, 43, {}, None)
        cls.resource_conn = mock_conn
        cls.conn = mock_client_conn
        with patch('awslimitchecker.services.ec2.logger') as mock_logger:
            cls._find_usage_spot_fleets()
        assert mock_conn.mock_calls == []
        assert mock_client_conn.mock_calls == [
            call.describe_spot_fleet_requests()
        ]

        total = cls.limits['Max active spot fleets per '
                           'region'].get_current_usage()
        assert len(total) == 1
        assert total[0].get_value() == 2

        totalcap = cls.limits['Max target capacity for all spot fleets '
                              'in region'].get_current_usage()
        assert len(totalcap) == 1
        assert totalcap[0].get_value() == 44

        cap_per_fleet = cls.limits['Max target capacity per spot '
                                   'fleet'].get_current_usage()
        assert len(cap_per_fleet) == 2
        assert cap_per_fleet[0].get_value() == 11
        assert cap_per_fleet[0].resource_id == 'req2'
        assert cap_per_fleet[1].get_value() == 33
        assert cap_per_fleet[1].resource_id == 'req4'

        launch_specs = cls.limits['Max launch specifications '
                                  'per spot fleet'].get_current_usage()
        assert len(launch_specs) == 2
        assert launch_specs[0].get_value() == 3
        assert launch_specs[0].resource_id == 'req2'
        assert launch_specs[1].get_value() == 1
        assert launch_specs[1].resource_id == 'req4'

        assert mock_logger.mock_calls == [
            call.debug('Getting spot fleet request usage'),
            call.error('Error: describe_spot_fleet_requests() response '
                       'includes pagination token, but pagination not '
                       'configured in awslimitchecker.'),
            call.debug('Skipping spot fleet request %s in state %s', 'req1',
                       'failed'),
            call.debug('Skipping spot fleet request %s in state %s',
                       'req3', 'modifying')
        ]

    def test_unsupported(self):
        mock_client_conn = Mock()
        err = botocore.exceptions.ClientError(
            {'Error': {'Code': 'UnsupportedOperation'}},
            'operation',
        )
        mock_client_conn.describe_spot_fleet_requests.side_effect = err
        cls = _Ec2Service(21, 43, {}, None)
        cls.conn = mock_client_conn
        cls._find_usage_spot_fleets()
        total = cls.limits['Max active spot fleets per '
                           'region'].get_current_usage()
        assert len(total) == 0

    def test_unknown_code(self):
        mock_client_conn = Mock()
        err = botocore.exceptions.ClientError(
            {'Error': {'Code': 'SomeCode'}},
            'operation',
        )
        mock_client_conn.describe_spot_fleet_requests.side_effect = err
        cls = _Ec2Service(21, 43, {}, None)
        cls.conn = mock_client_conn
        with pytest.raises(botocore.exceptions.ClientError):
            cls._find_usage_spot_fleets()

    def test_unknown_error(self):
        mock_client_conn = Mock()
        mock_client_conn.describe_spot_fleet_requests.side_effect = RuntimeError
        cls = _Ec2Service(21, 43, {}, None)
        cls.conn = mock_client_conn
        with pytest.raises(RuntimeError):
            cls._find_usage_spot_fleets()


class TestUpdateLimitsFromApi(object):

    def test_happy_path(self):
        data = fixtures.test_update_limits_from_api
        mock_conn = Mock()
        mock_client_conn = Mock()
        mock_client_conn.describe_account_attributes.return_value = data

        with patch(
                '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
        ) as m_use_vcpu:
            m_use_vcpu.return_value = False
            cls = _Ec2Service(21, 43, {}, None)
            cls.resource_conn = mock_conn
            cls.conn = mock_client_conn
            with patch('awslimitchecker.services.ec2.logger') as mock_logger:
                cls._update_limits_from_api()
        assert mock_conn.mock_calls == []
        assert mock_client_conn.mock_calls == [
            call.describe_account_attributes()
        ]
        assert mock_logger.mock_calls == [
            call.info("Querying EC2 DescribeAccountAttributes for limits"),
            call.debug('Done setting limits from API')
        ]
        assert cls.limits['Elastic IP addresses (EIPs)'].api_limit == 40
        assert cls.limits['Running On-Demand EC2 instances'].api_limit == 400
        assert cls.limits['VPC Elastic IP addresses (EIPs)'].api_limit == 200
        assert cls.limits['VPC security groups per elastic '
                          'network interface'].api_limit == 5

    def test_vcpu(self):
        data = fixtures.test_update_limits_from_api_vcpu
        mock_conn = Mock()
        mock_client_conn = Mock()
        mock_client_conn.describe_account_attributes.return_value = data

        with patch(
                '%s._use_vcpu_limits' % pb, new_callable=PropertyMock
        ) as m_use_vcpu:
            m_use_vcpu.return_value = False
            cls = _Ec2Service(21, 43, {}, None)
            cls.resource_conn = mock_conn
            cls.conn = mock_client_conn
            with patch('awslimitchecker.services.ec2.logger') as mock_logger:
                cls._update_limits_from_api()
        assert mock_conn.mock_calls == []
        assert mock_client_conn.mock_calls == [
            call.describe_account_attributes()
        ]
        assert mock_logger.mock_calls == [
            call.info("Querying EC2 DescribeAccountAttributes for limits"),
            call.debug('Done setting limits from API')
        ]
        assert cls.limits['Elastic IP addresses (EIPs)'].api_limit == 40
        assert cls.limits['VPC Elastic IP addresses (EIPs)'].api_limit == 200
        assert cls.limits['VPC security groups per elastic '
                          'network interface'].api_limit == 5

    def test_unsupported(self):
        data = fixtures.test_update_limits_from_api_unsupported
        mock_client_conn = Mock()
        mock_client_conn.describe_account_attributes.return_value = data

        cls = _Ec2Service(21, 43, {}, None)
        cls.conn = mock_client_conn
        cls._update_limits_from_api()
        lim = cls.limits['Elastic IP addresses (EIPs)']
        usage = lim.get_current_usage()
        assert len(usage) == 0


class TestUseVcpuLimits(object):

    @patch.dict(
        os.environ,
        {},
        clear=True
    )
    def test_useast1(self):
        with patch('%s.get_limits' % pb):
            cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='us-east-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is True
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {},
        clear=True
    )
    def test_beijing(self):
        with patch('%s.get_limits' % pb):
            cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='cn-north-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is False
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {},
        clear=True
    )
    def test_ningxia(self):
        with patch('%s.get_limits' % pb):
            cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='cn-northwest-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is False
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {},
        clear=True
    )
    def test_gov_west1(self):
        with patch('%s.get_limits' % pb):
            cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='us-gov-west-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is False
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {'USE_VCPU_LIMITS': 'true'},
        clear=True
    )
    def test_useast1_env_true(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='us-east-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is True
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {'USE_VCPU_LIMITS': 'true'},
        clear=True
    )
    def test_beijing_env_true(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='cn-north-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is True
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {'USE_VCPU_LIMITS': 'true'},
        clear=True
    )
    def test_ningxia_env_true(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='cn-northwest-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is True
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {'USE_VCPU_LIMITS': 'true'},
        clear=True
    )
    def test_gov_west1_env_true(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='us-gov-west-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is True
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {'USE_VCPU_LIMITS': 'false'},
        clear=True
    )
    def test_useast1_env_false(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='us-east-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is False
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {'USE_VCPU_LIMITS': 'false'},
        clear=True
    )
    def test_beijing_env_false(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='cn-north-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is False
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {'USE_VCPU_LIMITS': 'false'},
        clear=True
    )
    def test_ningxia_env_false(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='cn-northwest-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is False
        assert cls.conn == mock_orig_conn

    @patch.dict(
        os.environ,
        {'USE_VCPU_LIMITS': 'false'},
        clear=True
    )
    def test_gov_west1_env_false(self):
        cls = _Ec2Service(21, 43, {}, None)
        mock_orig_conn = Mock()
        cls.conn = mock_orig_conn

        def se_conn(klass):
            mock_conn = Mock()
            mock_conf = Mock()
            type(mock_conf).region_name = PropertyMock(
                return_value='us-gov-west-1'
            )
            type(mock_conn)._client_config = PropertyMock(
                return_value=mock_conf
            )
            klass.conn = mock_conn

        with patch('%s.connect' % pb, autospec=True) as m_connect:
            m_connect.side_effect = se_conn
            res = cls._use_vcpu_limits
        assert res is False
        assert cls.conn == mock_orig_conn



"""
awslimitchecker/tests/services/Route53.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2018 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

import sys
from awslimitchecker.tests.services import result_fixtures
from awslimitchecker.services.route53 import _Route53Service

# https://code.google.com/p/mock/issues/detail?id=249
# py>=3.4 should use unittest.mock not the mock package on pypi
if (
        sys.version_info[0] < 3 or
        sys.version_info[0] == 3 and sys.version_info[1] < 4
):
    from mock import patch, call, Mock, DEFAULT
else:
    from unittest.mock import patch, call, Mock, DEFAULT

pbm = 'awslimitchecker.services.route53'  # module patch base
pb = '%s._Route53Service' % pbm  # class patch pase


class Test_Route53Service(object):
    def _mock_get_hosted_zone_limit(self, Type, HostedZoneId):
        if HostedZoneId in result_fixtures.Route53.test_get_hosted_zone_limit:
            return result_fixtures.Route53.test_get_hosted_zone_limit[
                HostedZoneId][Type]

    def _mock_reponse_init(self, cls):
        response = result_fixtures.Route53.test_get_hosted_zones
        mock_conn = Mock()
        mock_conn.list_hosted_zones.return_value = response
        mock_conn.get_hosted_zone_limit = self._mock_get_hosted_zone_limit
        cls.conn = mock_conn

    def test_init(self):
        """test __init__()"""
        cls = _Route53Service(21, 43, {}, None)
        assert cls.service_name == 'Route53'
        assert cls.api_name == 'route53'
        assert cls.conn is None
        assert cls.warning_threshold == 21
        assert cls.critical_threshold == 43

    def test_get_limits(self):
        """
        Test that get_limits returns AwsLimits
        """
        cls = _Route53Service(21, 43, {}, None)
        cls.limits = {}
        res = cls.get_limits()
        assert sorted(res.keys()) == sorted([
            'Record sets per hosted zone',
            'VPC associations per hosted zone',
        ])
        for name, limit in res.items():
            assert limit.service == cls
            assert limit.def_warning_threshold == 21
            assert limit.def_critical_threshold == 43

    def test_find_usage(self):
        cls = _Route53Service(21, 43, {}, None)

        assert cls._have_usage is False
        cls.find_usage()
        assert cls._have_usage is True

    def test_update_limits_from_api(self):
        """test _update_limits_from_api method calls other methods"""

        mock_conn = Mock()
        with patch('%s.connect' % pb) as mock_connect:
            with patch.multiple(
                    pb,
                    _find_limit_hosted_zone=DEFAULT,
            ) as mocks:
                cls = _Route53Service(21, 43, {}, None)
                cls.conn = mock_conn
                cls._update_limits_from_api()
        assert mock_connect.mock_calls == [call()]
        assert mock_conn.mock_calls == []
        for x in [
            '_find_limit_hosted_zone',
        ]:
            assert mocks[x].mock_calls == [call()]

    def test_find_limit_hosted_zone_recordsets(self):
        cls = _Route53Service(21, 43, {}, None)
        self._mock_reponse_init(cls)
        cls._find_limit_hosted_zone()

        limit_key = cls.MAX_RRSETS_BY_ZONE["name"]
        assert cls.limits[limit_key].default_limit == 10000

        usage1 = cls.limits[limit_key].get_current_usage()[0]
        assert usage1.get_value() == 7500
        assert usage1.resource_id == "abc.example.com."
        assert usage1.get_maximum() == 10000

        usage2 = cls.limits[limit_key].get_current_usage()[1]
        assert usage2.get_value() == 2500
        assert usage2.resource_id == "def.example.com."
        assert usage2.get_maximum() == 10001

        usage2 = cls.limits[limit_key].get_current_usage()[2]
        assert usage2.get_value() == 5678
        assert usage2.resource_id == "ghi.example.com."
        assert usage2.get_maximum() == 10002

    def test_find_limit_hosted_zone_vpc_associations(self):
        cls = _Route53Service(21, 43, {}, None)
        self._mock_reponse_init(cls)
        cls._find_limit_hosted_zone()

        limit_key = cls.MAX_VPCS_ASSOCIATED_BY_ZONE["name"]
        assert cls.limits[limit_key].default_limit == 100

        usage1 = cls.limits[limit_key].get_current_usage()[0]
        assert usage1.get_value() == 10
        assert usage1.resource_id == "abc.example.com."
        assert usage1.get_maximum() == 100

        usage2 = cls.limits[limit_key].get_current_usage()[1]
        assert usage2.get_value() == 2
        assert usage2.resource_id == "def.example.com."
        assert usage2.get_maximum() == 101

    def test_required_iam_permissions(self):
        cls = _Route53Service(21, 43, {}, None)
        assert cls.required_iam_permissions() == [
            "route53:GetHostedZone",
            "route53:GetHostedZoneLimit",
            "route53:ListHostedZones"
        ]



"""
awslimitchecker/tests/metrics/test_dummy.py

The latest version of this package is available at:
<https://github.com/jantman/awslimitchecker>

################################################################################
Copyright 2015-2019 Jason Antman <jason@jasonantman.com>

    This file is part of awslimitchecker, also known as awslimitchecker.

    awslimitchecker is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    awslimitchecker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with awslimitchecker.  If not, see <http://www.gnu.org/licenses/>.

The Copyright and Authors attributions contained herein may not be removed or
otherwise altered, except to add the Author attribution of a contributor to
this work. (Additional Terms pursuant to Section 7b of the AGPL v3)
################################################################################
While not legally required, I sincerely request that anyone who finds
bugs please submit them at <https://github.com/jantman/awslimitchecker> or
to me via email, and that you send any contributions or improvements
either as a pull request on GitHub, or to me via email.
################################################################################

AUTHORS:
Jason Antman <jason@jasonantman.com> <http://www.jasonantman.com>
################################################################################
"""

import sys
from awslimitchecker.metrics import Dummy

if (
        sys.version_info[0] < 3 or
        sys.version_info[0] == 3 and sys.version_info[1] < 4
):
    from mock import Mock
else:
    from unittest.mock import Mock


class TestDummyInit(object):

    def test_happy_path(self):
        cls = Dummy('foo')
        assert cls._region_name == 'foo'
        assert cls._duration == 0.0
        assert cls._limits == []

    def test_flush(self, capsys):
        cls = Dummy('foo')
        cls.set_run_duration(123.45)
        limA = Mock(
            name='limitA', service=Mock(service_name='SVC1')
        )
        type(limA).name = 'limitA'
        limA.get_current_usage.return_value = []
        limA.get_limit.return_value = None
        cls.add_limit(limA)
        limB = Mock(
            name='limitB', service=Mock(service_name='SVC1')
        )
        type(limB).name = 'limitB'
        mocku = Mock()
        mocku.get_value.return_value = 6
        limB.get_current_usage.return_value = [mocku]
        limB.get_limit.return_value = 10
        cls.add_limit(limB)
        cls.flush()
        out, err = capsys.readouterr()
        assert err == ''
        assert out == 'DummyMetrics Provider flush for region=foo\n' \
                      'Duration: 123.45\n' \
                      'SVC1 / limitA: limit=unknown max_usage=0\n' \
                      'SVC1 / limitB: limit=10 max_usage=6\n'
